// constants
const MONTHLY_LABEL = "      -";
const COMPLETED_LABEL = "      |";
const OCCURRENCE_ONCE_LABEL = "     x";
const SHOW_IN_TOOLTIP_LABEL = "          x";

const REMINDER_ID_REF = "idRef";

var g_modified = false;

var	selectCalendarSync = false;
var calendarReminderArray = new Array(12);

var lastSearchString = null;

function populateDateList() {
	var currentDate = oeDatePicker.gSelectedDate;
	if ( currentDate == null ) {
		currentDate = new Date();
	}

	var year = parseInt(currentDate.getFullYear());

	var dayList = document.getElementById('daylist');
	var items = dayList.firstChild.childNodes;
	// remove ending nodes that could possibly be different each month
	// (i.e. any date greater than 28)
	var length = dayList.firstChild.childNodes.length - 1;	
	for ( var j = length; j >= 28; j-- ) {
		dayList.firstChild.removeChild( items[j] );
	}
	
	
	// now add proper # of days for this month
	var dayArray;
	if (reminderFox_isLeapYear(parseInt(year))) {
		dayArray = REMINDER_FOX_lDOMonth;
	}
	else {
		dayArray = REMINDER_FOX_DOMonth;
	}

	var monthlist = document.getElementById('monthlist');
	var currentMonth = reminderFox_getMonthAsInteger(monthlist.label);
	// if it's the Monthly special-case, treat it as a full month (31 days)
	if ( currentMonth == REMINDER_FOX_MONTHLY_WILDCARD_CHAR ) {
		currentMonth = 0;  
	}

	var numOfDays = dayArray[currentMonth];
 	for ( var k = 29; k < numOfDays + 1; k++ ) {
		var newDay = document.createElement("menuitem");
		var str = k + "";
		newDay.setAttribute("label", str);
		dayList.firstChild.appendChild(newDay);
	}
	

	// now make sure the dateList is not showing a date greater than
	// the number of days in the selected month; if it is set the list
	// to show the last day of the month
	var dayname = parseInt(dayList.label);

	var numOfDaysInt = parseInt( numOfDays );
	if ( dayname > numOfDaysInt ) {
		dayList.selectedItem = dayList.firstChild.childNodes[numOfDaysInt - 1];
	}
}


function populateDateListAndSelectNewReminderText() { 
	populateDateList();
	selectNewReminderText();
}

function selectTab() {
	// give focus to the reminder/todo description when switching tabs
	var remindersTab = document.getElementById("remindersTab");
	if ( remindersTab.selected ) {
		selectNewReminderText();
	}
	else {
		var todosTab = document.getElementById("todosTab");
		if ( todosTab.selected ) {
			var desc = document.getElementById("newTodoText");
	    	desc.focus();
	    }
	}
	
}
	
function loadReminders() {
	fillList(true, true);
	
	// select appropriate tab
	var currentTab = window.arguments[0];
	if ( currentTab == "todos" ) {
		var todosTab = document.getElementById("todosTab");
		var tabbox = document.getElementById("tabbox");
		tabbox.selectedTab = todosTab;		
	}
	
   	var todaysDate = new Date();
	oeDatePicker.onpopupshowing(todaysDate);
	selectCalendarSync = true;
	
	// show calendar based on button state
	var calendar = document.getElementById("oe-date-picker-overlay-box");
	var calendarToggleButton= document.getElementById("calendarToggle");
	if ( calendarToggleButton.checked ) {
		calendarToggleButton.setAttribute("tooltiptext", "Hide Calendar");	
		calendar.setAttribute("hidden", "false");	
	}
	else {
		calendarToggleButton.setAttribute("tooltiptext", "Show Calendar");	
		calendar.setAttribute("hidden", "true");	
	}

	
}

function fillList(  fillReminders,  fillTodoList) {
	var i;
	if ( fillReminders ) {
		var reminders = reminderFox_getReminderEvents();
		for( i = 0; i < reminders.length; i++) {
			createListItem(reminders[i], false);
		}
	}
	
	if ( fillTodoList ) {
		var todos = reminderFox_getReminderTodos();
		 for( i = 0; i < todos.length; i++) {
			createTodoListItem(todos[i], false);
		}
	}
		
	// now that we've added the sorted reminders in the list, let's
	// select (highlight) today's reminder or the next upcoming reminder
	highlightClosestUpcomingReminder();
	
	highlightTodo();
	
	// set the current date for adding new reminders
	var currentDate = new Date();
	setNewReminderDate(currentDate, false);

   	selectNewReminderText();
}

function doFindNext() {
	// only bring up find dialog if reminders tab is selected
	var remindersTab = document.getElementById("remindersTab");
	if ( !remindersTab.selected ) {
		return;
	}
	
	if ( lastSearchString == null ) {
		doFind();
	}
	else {
		findReminder();
	}
}

function doFind() {
	// only bring up find dialog if reminders tab is selected
	var remindersTab = document.getElementById("remindersTab");
	if ( !remindersTab.selected ) {
		return;
	}

	var nsIPromptService = Components.interfaces.nsIPromptService;
	var nsPrompt_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
	var gPromptService = Components.classes[nsPrompt_CONTRACTID].getService(nsIPromptService);
	var result = { value: lastSearchString };
	var dummy = { value: 0 };
	if (gPromptService.prompt(window,
                            "Find Reminder",
                            "Enter text to search for:",
                            result,
                            null,
                            dummy)) {
                            
            lastSearchString = result.value;
          
        	findReminder();             
    }    
}


function findReminder( ) {
	var i;
	var item;
	var row;
	var descCell;
	var descCellLabel;
	
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	// if nothing selected, default to start of list
	if ( selectedTreeItemIndex == -1 ) {
		selectedTreeItemIndex = 0;
	}
		
	var treeChildren = document.getElementById("treechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];

	var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	var descLabel = selectedTreeItem.childNodes[0].childNodes[1].getAttribute("label");
   caseInsensitiveLastSearchString = lastSearchString.toUpperCase();

	var treeitems = treeChildren.childNodes;
	var startIndex = selectedTreeItemIndex;
	if ( startIndex + 1 < treeitems.length ) {
		startIndex = startIndex + 1;
	}
	else { 
		startIndex = 0;
	}

	var foundIndex = -1;
	for ( i=startIndex; i < treeitems.length; i++) {
		item = treeitems[i];
		row = item.childNodes[0];
		descCell = row.childNodes[1];
		descCellLabel = descCell.getAttribute("label");
		descCellLabel = descCellLabel.toUpperCase();

		if ( descCellLabel.indexOf( caseInsensitiveLastSearchString ) != -1  ) {
			foundIndex = i;

			break;
		}
	}
	
	if ( foundIndex == -1 ) {
		for ( i=0; i < startIndex; i++) {
			item = treeitems[i];
			 row = item.childNodes[0];
			descCell = row.childNodes[1];
			descCellLabel = descCell.getAttribute("label");
			descCellLabel = descCellLabel.toUpperCase();

			if ( descCellLabel.indexOf( caseInsensitiveLastSearchString ) != -1  ) {
				foundIndex = i;
				break;
			}
		}
	}
	
	// if we found a search match, then highlight the new reminder in the list
	if ( foundIndex != -1 ) {		
		var treeSelection = document.getElementById("reminderTree").view.selection		
		treeSelection.select(foundIndex);
		
		// make sure that newly selected row is shown in scroll pane
		var value = document.getElementById("treeparent").value;
	  	var tree = document.getElementById("reminderTree");			
		var boxobject = tree.boxObject;
		boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
		
		// make sure it shows up at the top if possible
		var first = boxobject.getFirstVisibleRow();
		var last = boxobject.getLastVisibleRow();
		if ( foundIndex < first || foundIndex > last ) {				
			boxobject.scrollToRow(treeitems.length - 9);
		}
		
		var previousReminderOffset = 3;
		if ( foundIndex < previousReminderOffset ) { 
			boxobject.ensureRowIsVisible(i);
		}
		else {
			boxobject.ensureRowIsVisible(foundIndex  - previousReminderOffset);
		}
	}
}


function selectNewReminderText() {
    var desc = document.getElementById("newReminderText");  
    desc.focus();
}

function setNewReminderDate( date, repopulateDays ) {
	var month = date.getMonth();
	var day = date.getDate() - 1; // account for 1-offset
	

	var monthlist = document.getElementById('monthlist');
	var prefMonth = reminderFox_getMonthAsInteger(monthlist.label);

	// if the month has changed, we need to repopulate the day drop-down list
	if ( prefMonth != month ) {
		repopulateDays = true;
	}	
	
	var items = monthlist.firstChild.childNodes;
    monthlist.selectedItem = items[month];
    
    if ( repopulateDays ) {
   		populateDateList();
   	}
    
	var dayList = document.getElementById('daylist');
	items = dayList.firstChild.childNodes;
    dayList.selectedItem = items[day];	
}

function createMonthlyReminderListItems(reminder) {
	for ( var i = 0; i < 12; i++ ) {
		var monthlyReminder = reminderFox_cloneReminderFoxEvent(reminder);
		monthlyReminder.date.setMonth(i);
		createListItem( monthlyReminder, true );
	}
}

function toggleCalendar() {
	var calendar = document.getElementById("oe-date-picker-overlay-box");
	var calendarToggleButton= document.getElementById("calendarToggle");
	if ( calendarToggleButton.checked ) {
		calendar.setAttribute("hidden", "true");	
		calendarToggleButton.setAttribute("tooltiptext", "Show Calendar");	
	}
	else {
		calendar.setAttribute("hidden", "false");	
		calendarToggleButton.setAttribute("tooltiptext", "Hide Calendar");	
	}
}

function reminderSelected(event) {
	if ( selectCalendarSync ) {
		var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
		if ( selectedTreeItemIndex >= 0 ) {
			var treeChildren = document.getElementById("treechildren");
			if ( selectedTreeItemIndex >= treeChildren.childNodes.length ) {
				selectedTreeItemIndex = treeChildren.childNodes.length - 1;
			}
			var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
			var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
			var dateLabelArray = dateLabel.split(" ");
			var monthAsInteger = reminderFox_getMonthAsInteger(dateLabelArray[0]);		
			var day = dateLabelArray[1];
			var year = oeDatePicker.gSelectedDate.getFullYear();
			if ( year == null ) {
				year = new Date().getFullYear();
			}
			var newDate = new Date(year, monthAsInteger, day );	
			oeDatePicker.onpopupshowing(newDate);
		}
	}
}


function createCalendarReminderArray() {
	var monthArray;
	var monthDay;	
	var dayReminderArray;	
	var newDayReminder;
	var  length;
	var val;
		
	calendarReminderArray = new Array(12); 
	var reminders = reminderFox_getReminderEvents();	
	for( var i = 0; i < reminders.length; i++) {
		var reminder = reminders[i];
		var currentDate = oeDatePicker.gSelectedDate;
		if ( currentDate == null ) {
			currentDate = new Date();
		}
		reminder = reminderFox_processReminderDescription(reminder, currentDate.getFullYear());	
		// if it's marked as RemindUntilComplete, this should be treated as Today's date
	 	if ( reminder.remindUntilCompleted == REMINDERFOX_REMIND_UNTIL_COMPLETE_MARKED ) {
	 		reminder = reminderFox_cloneReminderFoxEvent(reminder);
	 		var todaysDate = new Date();
			reminder.date = todaysDate;
	 	}				
		if ( reminder.recurrence  == REMINDER_RECURRENCE_MONTHLY_DATE ) {
				for ( var k= 0; k < 12; k++ ) {
					var monthReplacement= k + "";
					var monthlyReminder = reminderFox_cloneReminderFoxEvent(reminder);
					monthlyReminder.date.setMonth( k );
					monthArray = calendarReminderArray[monthReplacement];
					if ( monthArray == null ) {
						monthArray = new Array(31);
						calendarReminderArray[monthReplacement] = monthArray;
					}
					
					monthDay = reminder.date.getDate();
					monthDay = reminderFox_getValidDateForMonth( new Date().getFullYear(), monthlyReminder.date.getMonth(), monthDay );
				
					dayReminderArray = monthArray[monthDay - 1];
					if ( dayReminderArray == null ) {
						dayReminderArray = new Array();
						monthArray[monthDay - 1] = dayReminderArray;
					}
					length = dayReminderArray.length;
					dayReminderArray[length] = monthlyReminder;
				}								
			}
		else {		
			monthArray = calendarReminderArray[reminder.date.getMonth()];
			if ( monthArray == null ) {
				monthArray = new Array(31);
				calendarReminderArray[reminder.date.getMonth()] = monthArray;
			}
				
			monthDay = reminder.date.getDate();
			dayReminderArray = monthArray[monthDay - 1];
			if ( dayReminderArray == null ) {
				dayReminderArray = new Array();
				monthArray[monthDay - 1] = dayReminderArray;
			}
			length = dayReminderArray.length;
			dayReminderArray[length] = reminder;
		}
	}
}



function createListItem(origReminder, isMonthly) {
	var treeSelection;
	var value ;
	var tree;	
	var boxobject;
	var first;
	var last;
	var previousReminderOffset ;	
	var reminder = reminderFox_cloneReminderFoxEvent(origReminder);
	
	var once = document.getElementById("occurrence");
	var treeChildren = document.getElementById("treechildren");
	var newItem = document.createElement("treeitem");
	var newRow = document.createElement("treerow");
	newRow.setAttribute( REMINDER_ID_REF, reminder.id );
	var newDateLabel = document.createElement("treecell");
	var newDescLabel = document.createElement("treecell");
	
	newItem.appendChild(newRow);
	newRow.appendChild(newDateLabel);
	newRow.appendChild(newDescLabel);
	var isImportant = false;
	
	
	// if it's a monthly reminder, add it for each month
	if ( !isMonthly && reminder.recurrence == REMINDER_RECURRENCE_MONTHLY_DATE ) {
		createMonthlyReminderListItems(reminder);
	}
	else {		
		var currentDate = oeDatePicker.gSelectedDate;
		if ( currentDate == null ) {
			currentDate = new Date();
		}
		reminder = reminderFox_processReminderDescription(reminder, currentDate.getFullYear());		
		// if it's marked as RemindUntilComplete, this should be treated as Today's date
	 	if ( reminder.remindUntilCompleted == REMINDERFOX_REMIND_UNTIL_COMPLETE_MARKED ) {
	 		var todaysDate = new Date();
	 		if (  reminder.recurrence == REMINDER_RECURRENCE_MONTHLY_DATE  ) {		
	 			// if it's a recurring reminder, only move the most recent occurrence to today's date 		
		 		if ( (reminder.date.getMonth() == todaysDate.getMonth() - 1 && reminder.date.getDate() > todaysDate.getDate()  ) || 
		 			 (reminder.date.getMonth() == todaysDate.getMonth()  && reminder.date.getDate() < todaysDate.getDate()  )	) {
					reminder.date = todaysDate;
				}
			}
			else {
				reminder.date = todaysDate;
			}			
	 	}
	
		var dayVal = reminder.date.getDate();
		var monthArray = calendarReminderArray[reminder.date.getMonth()];
		if ( monthArray == null ) {
			monthArray = new Array(31);
			calendarReminderArray[reminder.date.getMonth()] = monthArray;
		}


		if ( isMonthly ) {
			dayVal = reminderFox_getValidDateForMonth( new Date().getFullYear(), reminder.date.getMonth(), dayVal );
		}
		var monthDay = parseInt(dayVal);
	
		var dayReminderArray = monthArray[monthDay - 1];
		if ( dayReminderArray == null ) {
			dayReminderArray = new Array();
			monthArray[monthDay - 1] = dayReminderArray;

		}

		var  length = dayReminderArray.length;
		dayReminderArray[length] = reminder;;

		var monthAsText = reminderFox_getMonthAsText(reminder.date.getMonth());		
		newDateLabel.setAttribute("label", monthAsText + " " + dayVal);
		newDescLabel.setAttribute("label", reminder.summary);
		
		var newOccurrenceLabel = document.createElement("treecell");
		newRow.appendChild(newOccurrenceLabel);
		
		var monthlyLabel = document.createElement("treecell");
		newRow.appendChild(monthlyLabel);		
		
		var newCompleteLabel = document.createElement("treecell");
		newRow.appendChild(newCompleteLabel);
		
		var newRemindUntilCompletedLabel = document.createElement("treecell");
		newRow.appendChild(newRemindUntilCompletedLabel);
	
		// check for any options that are specified, and mark them in the columns 
		if ( reminder.recurrence == REMINDER_RECURRENCE_ONETIME ) {	
		        newOccurrenceLabel.setAttribute("label", OCCURRENCE_ONCE_LABEL);			
		}
		if ( reminder.priority == REMINDER_PRIORITY_IMPORTANT) { 
             // mark the date and description as important (red-highlighted)
            newDateLabel.setAttribute("properties", "important");
            newDescLabel.setAttribute("properties", "important");
            isImportant = true;
         }
         if ( reminder.complete ) {
        	 newCompleteLabel.setAttribute("label", COMPLETED_LABEL);		
         }
         if ( reminder.remindUntilCompleted != REMINDERFOX_REMIND_UNTIL_COMPLETE_NONE ) {
         	newRemindUntilCompletedLabel.setAttribute("label", COMPLETED_LABEL);			
         }
		

		if ( isMonthly ) {
			monthlyLabel.setAttribute("label", MONTHLY_LABEL);
		}
		
		// Figure out where to put the new item in sorted list
		var treeitems = treeChildren.childNodes;
		var added = false;
		for (var i=0; i < treeitems.length; i++) {
			var item = treeitems[i];
			var row = item.childNodes[0];
			var dateCell = row.childNodes[0];
			var dateLabel = dateCell.getAttribute("label");
			var dateLabelDiv = dateLabel.split(" ");
			var month = reminderFox_getMonthAsInteger(dateLabelDiv[0]);
			var date = dateLabelDiv[1];
			
			if (month > parseInt(reminder.date.getMonth()) || (month == parseInt(reminder.date.getMonth()) && parseInt(dayVal) <= date ) ) {	
				if (month == parseInt(reminder.date.getMonth()) && date == parseInt(dayVal)) {
					if ( !isImportant ) {
						// if this reminder is not marked as important, add to the 
						// end of the list of today's reminders
						continue;
					}
				}
				treeChildren.insertBefore(newItem, item);
				added = true;
				
				// highlight the newly added reminder in the list
				treeSelection = document.getElementById("reminderTree").view.selection;	
				if ( selectCalendarSync ) {
						treeSelection.select(i);
				}
				// make sure that newly selected row is shown in scroll pane
				value = document.getElementById("treeparent").value;
				tree = document.getElementById("reminderTree");					
				boxobject = tree.boxObject;
				boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
				
				first = boxobject.getFirstVisibleRow();
				last = boxobject.getLastVisibleRow();
				if ( i < first || i > last ) {	
					boxobject.scrollToRow(treeitems.length);
					boxobject.scrollToRow(treeitems.length - 9);
				}

							
				previousReminderOffset = 3;
				if ( i < previousReminderOffset ) { 
					boxobject.ensureRowIsVisible(i);
				}
				else {
					boxobject.ensureRowIsVisible(i  - previousReminderOffset);
				}
					
				break;
			}
		}
		
		if (!added){
			treeChildren.appendChild(newItem);
			// highlight the newly added reminder in the list
			treeSelection = document.getElementById("reminderTree").view.selection;	
			if ( selectCalendarSync ) {
				treeSelection.select(treeitems.length);
			}
			// make sure that newly selected row is shown in scroll pane
			value = document.getElementById("treeparent").value;
			tree = document.getElementById("reminderTree");					
			boxobject = tree.boxObject;
			boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
			
			first = boxobject.getFirstVisibleRow();
			last = boxobject.getLastVisibleRow();
			if ( i < first || i > last ) {	
				boxobject.scrollToRow(treeitems.length);			
				boxobject.scrollToRow(treeitems.length - 9);
			}
			
			previousReminderOffset = 3;
			if ( i < previousReminderOffset ) { 
				boxobject.ensureRowIsVisible(i);
			}
			else {
				boxobject.ensureRowIsVisible(i  - previousReminderOffset);
			}
			
		}
	}
}


function createTodoListItem(todo, sort) {
	var tree ;
	var treeSelection
	var boxobject;
	var item;	
	
	var treeChildren = document.getElementById("todoTreechildren");
	var newItem = document.createElement("treeitem");
	var newRow = document.createElement("treerow");
	newRow.setAttribute( REMINDER_ID_REF, todo.id );	
	var newDescLabel = document.createElement("treecell");
	
	newItem.appendChild(newRow);
	newRow.appendChild(newDescLabel);
	var isImportant = false;
	
	newDescLabel.setAttribute("label", todo.summary);
		
	var newCompletedLabel = document.createElement("treecell");
	newRow.appendChild(newCompletedLabel);
	
	// check for any options that are specified, and mark them in the columns 
	if ( todo.priority == REMINDER_PRIORITY_IMPORTANT) { 
            newDescLabel.setAttribute("properties", "important");
            isImportant = true;
     }
     if ( todo.showInTooltip ) {
            newCompletedLabel.setAttribute("label", SHOW_IN_TOOLTIP_LABEL);		
     }	
    
	var treeitems = treeChildren.childNodes;
	
	if( !sort || treeitems.length == 0 ) {
        treeChildren.appendChild(newItem);
        /*  
		// highlight the newly added reminder in the list 	     	
    	var treeSelection = document.getElementById("todoTree").view.selection;	
		treeSelection.select(treeitems.length - 1);
		
		var tree = document.getElementById("todoTree");					
		var boxobject = tree.boxObject;
		boxobject.ensureRowIsVisible(treeitems.length - 1);
		*/
	}
    // if current todo is not important, add after important items
    else if ( !isImportant) {
		var added = false;
		for (var i=0; i < treeitems.length && !added; i++) {
			item = treeitems[i];
			var row = item.childNodes[0];
			var todoRefId = row.getAttribute(REMINDER_ID_REF);			
			var todoAtIndex = reminderFox_getTodosById( todoRefId );				
			var descCell = row.childNodes[0];
			var descLabel = descCell.getAttribute("label");		
			
 			if ( !todoAtIndex.priority == REMINDER_PRIORITY_IMPORTANT ) {
	        	if (  todo.showInTooltip ) {
	     			treeChildren.insertBefore(newItem, item);
					added = true;	        	
					// highlight the newly added reminder in the list
					tree = document.getElementById("todoTree");	
					treeSelection = tree.view.selection;	
					treeSelection.select(i);					
	
					boxobject = tree.boxObject;
					boxobject.ensureRowIsVisible(i);
				}
	        	else {
	        		if ( !todoAtIndex.showInTooltip  ) {
						treeChildren.insertBefore(newItem, item);
						added = true;
						// highlight the newly added reminder in the list
						tree = document.getElementById("todoTree");	
						treeSelection = tree.view.selection;	
						treeSelection.select(i);
						
						boxobject = tree.boxObject;
						boxobject.ensureRowIsVisible(i);
	        		}
	        	}
	        
	        }
		}
		
    
    	if ( !added ) {
           	treeChildren.appendChild(newItem);
			// highlight the newly added reminder in the list 	 
	    	tree = document.getElementById("todoTree");	
	    	treeSelection = tree.view.selection;	
			treeSelection.select(treeitems.length);			
							
			boxobject = tree.boxObject;
			boxobject.ensureRowIsVisible(treeitems.length );

			/*
			// make sure that newly selected row is shown in scroll pane
			var value = document.getElementById("todoTreeparent").value;
			var tree = document.getElementById("todoTree");					
			var boxobject = tree.boxObject;
			boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
			
			var first = boxobject.getFirstVisibleRow();
			var last = boxobject.getLastVisibleRow();
			if ( i < first || i > last ) {	
				boxobject.scrollToRow(treeitems.length);			
				boxobject.scrollToRow(treeitems.length - 9);
			}
			
			var previousReminderOffset = 3;
			if ( i < previousReminderOffset ) { 
				boxobject.ensureRowIsVisible(i);
			}
			else {
				boxobject.ensureRowIsVisible(i  - previousReminderOffset);
			}
	    	*/
       }
    }
    // new Todo is important - add to top of list
    else {        	
    	item = treeitems[0];
    	treeChildren.insertBefore(newItem, item);
		added = true;
		
		// highlight the newly added reminder in the list
		tree = document.getElementById("todoTree");	
		treeSelection = tree.view.selection;	
		treeSelection.select(0);
						
		boxobject = tree.boxObject;
		boxobject.ensureRowIsVisible(0);
		/*
		// make sure that newly selected row is shown in scroll pane
		var value = document.getElementById("todoTreeparent").value;
		var tree = document.getElementById("todoTree");					
		var boxobject = tree.boxObject;
		boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
		
		var first = boxobject.getFirstVisibleRow();
		var last = boxobject.getLastVisibleRow();
		if ( i < first || i > last ) {	
			boxobject.scrollToRow(treeitems.length);
			boxobject.scrollToRow(treeitems.length - 9);
		}

					
		var previousReminderOffset = 3;
		if ( i < previousReminderOffset ) { 
			boxobject.ensureRowIsVisible(i);
		}
		else {
			boxobject.ensureRowIsVisible(i  - previousReminderOffset);
		}
		*/
    }	
	
}


function createTodoListItemAtIndex(todo, index) {
	var tree ;
	var treeSelection
	var boxobject;
	var item;	
	
	var treeChildren = document.getElementById("todoTreechildren");
	var newItem = document.createElement("treeitem");
	var newRow = document.createElement("treerow");
	newRow.setAttribute( REMINDER_ID_REF, todo.id );	
	var newDescLabel = document.createElement("treecell");
	
	newItem.appendChild(newRow);
	newRow.appendChild(newDescLabel);
	var isImportant = false;
	
	newDescLabel.setAttribute("label", todo.summary);
		
	var newCompletedLabel = document.createElement("treecell");
	newRow.appendChild(newCompletedLabel);
	
	// check for any options that are specified, and mark them in the columns 
	if ( todo.priority == REMINDER_PRIORITY_IMPORTANT) { 
            newDescLabel.setAttribute("properties", "important");
            isImportant = true;
     }
     if ( todo.showInTooltip ) {
            newCompletedLabel.setAttribute("label", SHOW_IN_TOOLTIP_LABEL);		
     }	
    
	var treeitems = treeChildren.childNodes;	
	// insert at end if last item
	if ( index >= treeitems.length ) {
        treeChildren.appendChild(newItem);
        // highlight the newly added reminder in the list 	 
        tree = document.getElementById("todoTree");	
        treeSelection = tree.view.selection;	
        treeSelection.select(treeitems.length );			
                        
        boxobject = tree.boxObject;
        boxobject.ensureRowIsVisible(treeitems.length );	
	
	}
	else {
		// otherwise insert before current item at index
		item = treeitems[index ];
		treeChildren.insertBefore(newItem, item);
		
		tree = document.getElementById("todoTree");	
		treeSelection = tree.view.selection;	
		treeSelection.select(index);					
	
		boxobject = tree.boxObject;
		boxobject.ensureRowIsVisible(index);	
	}
}


/**
  * select (highlight) today's reminder or the next upcoming reminder
  */
function highlightClosestUpcomingReminder() {
	var currentDate = new Date();
	var currentMonth = parseInt(currentDate.getMonth());
	var currentDay = parseInt(currentDate.getDate() );

	var treeChildren = document.getElementById("treechildren");
	var treeitems = treeChildren.childNodes;
	for (var i=0; i < treeitems.length; i++) {
		var item = treeitems[i];
		var row = item.childNodes[0];
		var dateCell = row.childNodes[0];
		var dateLabel = dateCell.getAttribute("label");
		var dateLabelDiv = dateLabel.split(" ");
		var month = reminderFox_getMonthAsInteger(dateLabelDiv[0]);
		var date = parseInt(dateLabelDiv[1]);
		if ( month > currentMonth || (month == currentMonth && date >= currentDay ) ) {
			var treeSelection = document.getElementById("reminderTree").view.selection		
			treeSelection.select(i);
			
			// make sure that newly selected row is shown in scroll pane
			var value = document.getElementById("treeparent").value;
		  	var tree = document.getElementById("reminderTree");			
			var boxobject = tree.boxObject;
			boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
			
			// make sure it shows up at the top if possible
			var first = boxobject.getFirstVisibleRow();
			var last = boxobject.getLastVisibleRow();
			if ( i < first || i > last ) {				
				boxobject.scrollToRow(treeitems.length - 9);
			}
			
			var previousReminderOffset = 3;
			if ( i < previousReminderOffset ) { 
				boxobject.ensureRowIsVisible(i);
			}
			else {
				boxobject.ensureRowIsVisible(i  - previousReminderOffset);
			}
			break;
		}
	}
}

/**
  * select (highlight) today's reminder or the next upcoming reminder
  */
function highlightReminderForDate(currentMonth, currentDay) {
	var highlighted = false;
	var treeChildren = document.getElementById("treechildren");
	var treeitems = treeChildren.childNodes;
	for (var i=0; i < treeitems.length; i++) {
		var item = treeitems[i];
		var row = item.childNodes[0];
		var dateCell = row.childNodes[0];
		var dateLabel = dateCell.getAttribute("label");
		var dateLabelDiv = dateLabel.split(" ");
		var month = reminderFox_getMonthAsInteger(dateLabelDiv[0]);
		var date = parseInt(dateLabelDiv[1]);
		if ( month == currentMonth && date == currentDay ) {
			highlighted = true;
			var treeSelection = document.getElementById("reminderTree").view.selection		
			treeSelection.select(i);

			// make sure that newly selected row is shown in scroll pane
			var value = document.getElementById("treeparent").value;
		  	var tree = document.getElementById("reminderTree");			
			var boxobject = tree.boxObject;
			boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
			
			// make sure it shows up at the top if possible
			var first = boxobject.getFirstVisibleRow();
			var last = boxobject.getLastVisibleRow();
			
			if ( i <= first || i >= last ) {				
				boxobject.scrollToRow(treeitems.length - 9);
			}
			
			var previousReminderOffset = 3;
			if ( i < previousReminderOffset ) { 
				boxobject.ensureRowIsVisible(i);
			}
			else {
				boxobject.ensureRowIsVisible(i  - previousReminderOffset);
			}
			break;
		}
	}
	
	if ( !highlighted ) {
		selectNewReminderText();
	}
}

function highlightTodo() {
	var tree = document.getElementById("todoTree");			
	var treeSelection = tree.view.selection;	
	treeSelection.select(0);	
	treeSelection.clearSelection();
			
	var boxobject = tree.boxObject;
	boxobject.ensureRowIsVisible(0);	
}


function removeListItemsForReminder(reminder) {
	var treeChildren = document.getElementById("treechildren");
	var treeitems = treeChildren.childNodes;
	for (var i=treeitems.length - 1; i >= 0; i--) {
		var item = treeitems[i];
		var row = item.childNodes[0];
		var reminderRefId = row.getAttribute(REMINDER_ID_REF);	
		if ( reminderRefId == reminder.id ) {			
			treeChildren.removeChild(item);
		}
	}
}

function removeTodoListItem(todo) {
	var treeChildren = document.getElementById("todoTreechildren");
	var treeitems = treeChildren.childNodes;

	for (var i=0; i < treeitems.length; i++) {
		var item = treeitems[i];
		var row = item.childNodes[0];
		var todoRefId = row.getAttribute(REMINDER_ID_REF);		
		if ( todoRefId == todo.id ) {			
			treeChildren.removeChild(item);
			break;
		}
	}	
}


// saves and closes the 'edit reminders' dialog
function revert() {
	// there were changes.  Prompt user to revert/cancel
	// get a reference to the prompt service component.
	var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
	                    .getService(Components.interfaces.nsIPromptService);

	var flags=promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_0 +
   	promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_1;     
	var msg =  "Are you sure you want to revert?  All of the changes made during this session will be lost.";
	var buttonPressed = promptService.confirmEx(window,"Revert changes?",
	  msg,
	  flags, "Revert", "Cancel", null, null, {});		  
	  
	// revert pressed
	if ( buttonPressed == 0 ) {		
		var revertReminders = false;
		var revertTodos = false;
//		if ( origReminders != reminders ) {
			revertReminders = true;
//		}
//		if ( origTodos != todos ) {
			revertTodos = true;
//		}
		reminderFox_clearRemindersAndTodos();

		removeAllListItems(revertReminders, revertTodos);
		
		fillList(revertReminders, revertTodos);
		refreshCalendar();
					
		// grey out the rever buttons
		var revertButton= document.getElementById("revertbutton");
		revertButton.setAttribute( "disabled", true );
		
		var revertButtonTodos= document.getElementById("revertbuttonTodos");
		revertButtonTodos.setAttribute( "disabled", true );
		
		g_modified = false;
	}	
}

function removeAllListItems(removeReminders, removeTodos) {
	var reminderTree = document.getElementById("reminderTree");
	var treeChildren = document.getElementById("treechildren");
	reminderTree.removeChild(treeChildren);
	
	var newchildren = document.createElement("treechildren");
	newchildren.setAttribute("id", "treechildren");
	newchildren.setAttribute("ondblclick", "if (event.button == 0) modifyReminderDescription()");
	reminderTree.appendChild(newchildren);

	var todoTreeChildren = document.getElementById("todoTreechildren");
	var treeitems = todoTreeChildren.childNodes;
	for (var i=treeitems.length-1; i >= 0; i--) {
		var item = treeitems[i];
		todoTreeChildren.removeChild(item);
	}		
}


// saves and closes the 'edit reminders' dialog
function saveSettings() {
	if ( g_modified ) {
		reminderFox_writeOutReminders();
		reminderFox_writeOutTodos();
			
		try {
			var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
			var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);	
			var windowEnumerator = windowManagerInterface.getEnumerator("navigator:browser");
		
			// update all of the browsers
			while (windowEnumerator.hasMoreElements()) {
				var currentWindow = windowEnumerator.getNext();		
				currentWindow.reminderFoxEvents = reminderFoxEvents;
				currentWindow.reminderFoxTodos  = reminderFoxTodos;
				
				currentWindow.reminderFox_loadReminderFox(true);			
				currentWindow.reminderFox_clearRemindersAndTodos();
			}
		}
		catch(e) {
		}
	}
	
	// clear reminders from memory
	reminderFox_clearRemindersAndTodos();
	
	g_modified = false;
	window.close();
}

// Adds the new reminder from the edit reminder dialog
function addReminder() {
	var monthName = document.getElementById("monthlist");
	var dayname = document.getElementById("daylist");
	var desc = document.getElementById("newReminderText");
	var once = document.getElementById("occurrence");
	var important = document.getElementById("important");
	
	if (desc.value == "")
		desc.value = "No Description";	

    // make sure there's no special (separator) characters being 
    // used; if so, replace them
    desc.value = desc.value.replace( /~/g, "-" );
    desc.value = desc.value.replace( /`/g, "'" );		
	
	var prefMonth = reminderFox_getMonthAsInteger(monthName.label);

	var date = new Date();
	date.setDate(dayname.label);
	if (prefMonth == REMINDER_FOX_MONTHLY_WILDCARD_CHAR ) {
		month = date.getMonth();
		date.setMonth(month);
	}
	else {
		date.setMonth(prefMonth);
	}

	var reminderEvent = new ReminderFoxEvent( date, desc.value );
	
	if ( prefMonth == REMINDER_FOX_MONTHLY_WILDCARD_CHAR ) {
		reminderEvent.recurrence = REMINDER_RECURRENCE_MONTHLY_DATE;
	}				
	
	// if this is marked as Important, denote that in the preferences
	if ( important.checked ) {
		reminderEvent.priority = REMINDER_PRIORITY_IMPORTANT;
	}		
	
	// if this is a one-time-only reminder, denote that in the preferences
	if ( once.checked && prefMonth != REMINDER_FOX_MONTHLY_WILDCARD_CHAR ) {
		reminderEvent.recurrence = REMINDER_RECURRENCE_ONETIME;
	}	

	// add reminder in sorted order...
	var sortedIndex = reminderFox_getSortedIndexOfNewReminder( reminderEvent, false );
	var reminders = reminderFox_getReminderEvents();	
	reminderFox_insertIntoArray( reminders, reminderEvent, sortedIndex );
	
	createListItem(reminderEvent, false);

	desc.value = null;

	var year = oeDatePicker.gSelectedDate.getFullYear();
	if ( year == null ) {
		year = new Date().getFullYear();
	}
	var newDate = new Date( year, reminderEvent.date.getMonth(), reminderEvent.date.getDate() );	
	oeDatePicker.onpopupshowing(newDate);
	
	modifiedReminders();
	
}

function modifiedReminders() {
	g_modified = true;

	var revertButton= document.getElementById("revertbutton");
	revertButton.setAttribute( "disabled", false );
	
	var revertButtonTodos= document.getElementById("revertbuttonTodos");
	revertButtonTodos.setAttribute( "disabled", false );
}

function addTodo() {
	var desc = document.getElementById("newTodoText");
	var important = document.getElementById("importantTodo");
	var showInTooltip = document.getElementById("showInTooltip");
	
	if (desc.value == "")
		desc.value = "No Description";

    // make sure there's no special (separator) characters being 
    // used; if so, replace them
    desc.value = desc.value.replace( /~/g, "-" );
    desc.value = desc.value.replace( /`/g, "'" );
	
	
	
	
	var newTodo = new ReminderFoxTodo( desc.value );

	// if this is marked as Important, denote that in the preferences
	if ( important.checked ) {
		newTodo.priority = REMINDER_PRIORITY_IMPORTANT;
	}		

	// if this is marked as Remind Until Completed, denote that in the preferences
	if ( showInTooltip.checked ) {
		newTodo.showInTooltip = true;
	}
	else {
		newTodo.showInTooltip = false;
	}
	
	// add todo in sorted order...
	var sortedIndex = reminderFox_getSortedIndexOfNewTodo( newTodo );
	var todos = reminderFox_getReminderTodos();
	reminderFox_insertIntoArray( todos, newTodo, sortedIndex );

	createTodoListItem(newTodo, true);
	desc.value = null;

	modifiedReminders();
}


function removeReminder() {
	var currentReminder = getReminderForSelectedItem( );

	// remove from UI list
	removeListItemsForReminder(currentReminder);
	
	// remove from model list
	var reminders = reminderFox_getReminderEvents();	
	for( var i = 0; i < reminders.length; i++) {
		if ( reminders[i].id == currentReminder.id ) {
			reminderFox_removeElement(reminders, i);
			break;
		}
	}
}


function removeTodo() {
	var currentTodo = getTodoForSelectedItem();
	
	// remove from UI list
	removeTodoListItem(currentTodo);
	
	// remove from model list
	var todos = reminderFox_getReminderTodos();
	for( var i = 0; i < todos.length; i++) {
		if ( todos[i].id == currentTodo.id ) {
			reminderFox_removeElement(todos, i);
			break;
		}
	}

}


function activateContext(event) {
	var tree = document.getElementById("reminderTree");
	var treeContextMenu = document.getElementById("treechildren-contextmenu");

	document.popupNode = tree;

	// modify context menu depending on Important reminder status
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	if ( selectedTreeItemIndex == -1 ) {
		return;
	}
	treeContextMenu.showPopup(tree, event.clientX, event.clientY, "bottomleft", "topleft");
	
	var currentReminder = getReminderForSelectedItem( );
	if ( currentReminder == null ) {
		return;
	}

	var contextMenuImportant = document.getElementById("treechildren-contextmenu-important");	
	if ( currentReminder.priority != REMINDER_PRIORITY_IMPORTANT ) {
		contextMenuImportant.setAttribute("checked", "false");	
	}
	else {
		contextMenuImportant.setAttribute("checked", "true");	
	}
	
	var contextMenuRemindComplete = document.getElementById("treechildren-contextmenu-remindUntilComplete");	
	if ( currentReminder.remindUntilCompleted != REMINDERFOX_REMIND_UNTIL_COMPLETE_NONE) {
		contextMenuRemindComplete.setAttribute("checked", "true");	 
	}
	else {
		contextMenuRemindComplete.setAttribute("checked", "false");	 
	}

	var contextMenuComplete = document.getElementById("treechildren-contextmenu-markAsComplete");	
	if ( currentReminder.complete ) {
			contextMenuComplete.setAttribute("checked", "true");	
	}
	else {
		contextMenuComplete.setAttribute("checked", "false");	
	}
	
}

function activateTodoContext(event) {
	var tree = document.getElementById("todoTree");
	var treeContextMenu = document.getElementById("todo-treechildren-contextmenu");
	document.popupNode = tree;
	
	// modify context menu depending on Important reminder status
	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	var treeChildren = document.getElementById("todoTreechildren");
	if ( treeChildren == null || selectedTreeItemIndex == -1 || treeChildren.childNodes.length == 0 ) {
		return;
	}
	treeContextMenu.showPopup(tree, event.clientX, event.clientY, "bottomleft", "topleft");
	
	var currentTodo = getTodoForSelectedItem( );
	if ( currentTodo == null ) {
		return;
	}
	
	var contextMenuImportant = document.getElementById("treechildren-contextmenu-important2");	
	if ( currentTodo.priority != REMINDER_PRIORITY_IMPORTANT ) {
		contextMenuImportant.setAttribute("checked", "false");	
	}
	else {
		contextMenuImportant.setAttribute("checked", "true");	
	}	


	var contextMenuShowInTooltip = document.getElementById("treechildren-contextmenu-showInTooltip");	
	if ( currentTodo.showInTooltip ) {
		contextMenuShowInTooltip.setAttribute("checked", "true");	
	}
	else {
		contextMenuShowInTooltip.setAttribute("checked", "false");	
	}

	// disable moveUp/moveDown if that option isn't available
	var moveUpMenu = document.getElementById("treechildren-contextmenu-moveUp");	
	if ( selectedTreeItemIndex == 0 ) {
		moveUpMenu.setAttribute( "disabled", true );
	}
	else {
		moveUpMenu.setAttribute( "disabled", false );
	}
	
	var moveDownMenu = document.getElementById("treechildren-contextmenu-moveDown");	
	if ( selectedTreeItemIndex == (treeChildren.childNodes.length - 1) ) {
		moveDownMenu.setAttribute( "disabled", true );
	}
	else {
		moveDownMenu.setAttribute( "disabled", false );
	}	
}


function processKeyPressedOnReminderTree(event) {
	if (event.keyCode == 46)
		userDeleteReminder();
}
function processKeyPressedOnTodoTree(event) {
	if (event.keyCode == 46)
		userDeleteTodo();
}


function modifyTodoDescription()
{
	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	var treeChildren = document.getElementById("todoTreechildren");
	if ( selectedTreeItemIndex == -1 ) {
		return;
	}
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	if ( selectedTreeItem == null ) {
		return;
	}
	var descLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");

	var nsIPromptService = Components.interfaces.nsIPromptService;
	var nsPrompt_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
	var gPromptService = Components.classes[nsPrompt_CONTRACTID].getService(nsIPromptService);
	var result = { value: descLabel };
	var dummy = { value: 0 };
	if (gPromptService.prompt(window,
                            "Description",
                            "Modify description for ToDo:",
                            result,
                            null,
                            dummy)) {
    
    
		var currentTodo = getTodoForSelectedItem();
		var summaryValue = result.value;
		
	    // make sure there's no special (separator) characters being 
	    // used; if so, replace them
	    summaryValue= summaryValue.replace( /~/g, "-" );
	    summaryValue = summaryValue.replace( /`/g, "'" );				
	    
   		currentTodo.summary = summaryValue;
		
		var modified = updateTodoInList( currentTodo );

		refreshCalendar();
		
		modifiedReminders();    
  }
}



function modifyReminderDescription() {
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	var treeChildren = document.getElementById("treechildren");
	 
	if ( selectedTreeItemIndex == -1 ) {
		return;
	}
	
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	if (  selectedTreeItem == null ) {
		return;
	}
	var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	var currentReminder = getReminderForSelectedItem();

	var nsIPromptService = Components.interfaces.nsIPromptService;
	var nsPrompt_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
	var gPromptService = Components.classes[nsPrompt_CONTRACTID].getService(nsIPromptService);
	var result = { value: currentReminder.summary };
	var dummy = { value: 0 };
	if (gPromptService.prompt(window,
                            "Description",
                            "Modify description for " + dateLabel + " reminder:",
                            result,
                            null,
                            dummy)) {

		var summaryValue = result.value;
		
	    // make sure there's no special (separator) characters being 
	    // used; if so, replace them
	    summaryValue= summaryValue.replace( /~/g, "-" );
	    summaryValue = summaryValue.replace( /`/g, "'" );				
	    
   		currentReminder.summary = summaryValue;

		var currentDate = oeDatePicker.gSelectedDate;
		if ( currentDate == null ) {
			currentDate = new Date();
		}	
		var reminder = reminderFox_processReminderDescription(currentReminder,  currentDate.getFullYear());				
		var modified = updateReminderInList( reminder );

		refreshCalendar();
		
		modifiedReminders();
  	}
}

function userDeleteReminder() {
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	var treeChildren = document.getElementById("treechildren");
	if ( selectedTreeItemIndex < 0 || selectedTreeItemIndex >= treeChildren.childNodes.length ) {
		return;
	}
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];

	var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	var descLabel = selectedTreeItem.childNodes[0].childNodes[1].getAttribute("label");
	
	var nsIPromptService = Components.interfaces.nsIPromptService;
	var nsPrompt_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
	var gPromptService = Components.classes[nsPrompt_CONTRACTID].getService(nsIPromptService);
	if (gPromptService.confirm(window,
                            "Delete Reminder",
                            "Press OK to delete the reminder\n  " + dateLabel + ": " + descLabel   )) {
     	removeReminder();
     	
		refreshCalendar();
		
		modifiedReminders();
     }
}

function userMoveReminderToTodo() {
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	var treeChildren = document.getElementById("treechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];

	var dateLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	var descLabel = selectedTreeItem.childNodes[0].childNodes[1].getAttribute("label");
	
	var nsIPromptService = Components.interfaces.nsIPromptService;
	var nsPrompt_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
	var gPromptService = Components.classes[nsPrompt_CONTRACTID].getService(nsIPromptService);
	if (gPromptService.confirm(window,
                            "Move to ToDo",
                            "Press OK to move the following reminder to the ToDo list:\n  " + descLabel   )) {

			var currentReminder = getReminderForSelectedItem( );
			var newTodo = new ReminderFoxTodo( currentReminder.summary );
			if ( currentReminder.priority ==  REMINDER_PRIORITY_IMPORTANT ) {
				newTodo.priority = REMINDER_PRIORITY_IMPORTANT;
			}
			
			// remove from reminders
			removeReminder();
			
			// add todo in sorted order...
			var sortedIndex = reminderFox_getSortedIndexOfNewTodo( newTodo );
			var todos = reminderFox_getReminderTodos();
			reminderFox_insertIntoArray( todos, newTodo, sortedIndex );
		
			createTodoListItem(newTodo, true);
	
	    	refreshCalendar();
	    
    		modifiedReminders();	    	
  	}
}

function userDeleteTodo() {
	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	var treeChildren = document.getElementById("todoTreechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];

	var descLabel = selectedTreeItem.childNodes[0].childNodes[0].getAttribute("label");
	
	var nsIPromptService = Components.interfaces.nsIPromptService;
	var nsPrompt_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
	var gPromptService = Components.classes[nsPrompt_CONTRACTID].getService(nsIPromptService);
	if (gPromptService.confirm(window,
                            "Delete ToDo",
                            "Press OK to delete the ToDo:\n  " + descLabel   )) {
     	removeTodo();
     	
  		modifiedReminders();
     }
}





function moveTodoUp() {
	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	var currentTodo = getTodoForSelectedItem( );

	var todos = reminderFox_getReminderTodos();
	var i;
	var found = false;
	for (  i = 0; i < todos.length; i++ ) {
		if ( todos[i].id == currentTodo.id ) {
			found = true;
			break;
		}
	}
	
	if ( found && i > 0 ) { 
		var newIndex = i - 1;  // insert before old position
		
		var newTodo =	reminderFox_cloneReminderFoxTodo( currentTodo );
		removeTodo();
		
		reminderFox_insertIntoArray( todos, newTodo, newIndex );
	
		var newListIndex = selectedTreeItemIndex - 1;
		createTodoListItemAtIndex(newTodo,newListIndex );
		
		modifiedReminders();
	}
}



function moveTodoDown() {	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	var currentTodo = getTodoForSelectedItem( );

	var todos = reminderFox_getReminderTodos();
	var i;
	var found = false;
	for (  i = 0; i < todos.length; i++ ) {
		if ( todos[i].id == currentTodo.id ) {
			found = true;
			break;
		}
	}
	
	if ( found ) { 
		var newIndex = i + 1;  // insert after old position
		
		var newTodo =	reminderFox_cloneReminderFoxTodo( currentTodo );
		removeTodo();
		
		reminderFox_insertIntoArray( todos, newTodo, newIndex );
	
		var newListIndex = selectedTreeItemIndex + 1;
		createTodoListItemAtIndex(newTodo,newListIndex );
		
		modifiedReminders();
	}
}


function toggleTodoImportantFlag() {
	var currentTodo = getTodoForSelectedItem( );
	if ( currentTodo.priority == REMINDER_PRIORITY_NORMAL) { 
		currentTodo.priority = REMINDER_PRIORITY_IMPORTANT;
	}
	else {
		currentTodo.priority = REMINDER_PRIORITY_NORMAL;
	}
	
	var modified = updateTodoInList( currentTodo );
	refreshCalendar();
}

function getReminderForSelectedItem() {
	var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
	var treeChildren = document.getElementById("treechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	var reminderRefId = selectedTreeItem.childNodes[0].getAttribute(REMINDER_ID_REF);
	return reminderFox_getRemindersById( reminderRefId );
}


function getTodoForSelectedItem() {
	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	var treeChildren = document.getElementById("todoTreechildren");
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];
	var todoRefId = selectedTreeItem.childNodes[0].getAttribute(REMINDER_ID_REF);
	return reminderFox_getTodosById( todoRefId );
}

function toggleImportantFlag() {
	var currentReminder = getReminderForSelectedItem( );
	if ( currentReminder.priority == REMINDER_PRIORITY_NORMAL) { 
		currentReminder.priority = REMINDER_PRIORITY_IMPORTANT;
	}
	else {
		currentReminder.priority = REMINDER_PRIORITY_NORMAL;
	}
	var modified = updateReminderInList( currentReminder );
	refreshCalendar();
}

function updateReminderInList( reminder) {
	var modified = true;
	var treeChildren = document.getElementById("treechildren");
	
	if ( reminder.recurrence  == REMINDER_RECURRENCE_MONTHLY_DATE ) {
		var treeitems = treeChildren.childNodes;
		for (var i=0; i < treeitems.length; i++) {
			var item = treeitems[i];		
			var row = item.childNodes[0];
			var reminderRefId = row.getAttribute(REMINDER_ID_REF);	
			if ( reminderRefId == reminder.id ) {
				updateReminder(  reminder, item );
			}
		}
	}
	else {
		var selectedTreeItemIndex = document.getElementById("reminderTree").currentIndex;
		var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];	
		updateReminder(  reminder, selectedTreeItem );
	}
	
	if ( modified ) {
		modifiedReminders();	
	}
	return modified;
}

function updateReminder( reminder, selectedTreeItem) {
	var modified = true;

	var dateNode = selectedTreeItem.childNodes[0].childNodes[0];
	var descriptionNode = selectedTreeItem.childNodes[0].childNodes[1];
	
	// update description
	
	var year = oeDatePicker.gSelectedDate.getFullYear();
	if ( year == null ) {
		year = new Date().getFullYear();
	}

	reminder = reminderFox_processReminderDescription(reminder, year);	
	descriptionNode.setAttribute( "label", reminder.summary );
	
	 // update importance
	 if ( reminder.priority == REMINDER_PRIORITY_IMPORTANT) { 
             // mark the date and description as important (red-highlighted)
            dateNode.setAttribute("properties", "important");
            descriptionNode.setAttribute("properties", "important");
         }
       else {
            dateNode.removeAttribute("properties");
            descriptionNode.removeAttribute("properties");
       }
		
		// update once
		var oneTimeNode = selectedTreeItem.childNodes[0].childNodes[2];		
		if (  reminder.recurrence == REMINDER_RECURRENCE_ONETIME ) {
				oneTimeNode.setAttribute("label", OCCURRENCE_ONCE_LABEL);
		}  
		else {
				oneTimeNode.removeAttribute("label");
		}
		
		
  		// update monthly
      	var monthNode = selectedTreeItem.childNodes[0].childNodes[3];  		
		if (  reminder.recurrence == REMINDER_RECURRENCE_MONTHLY_DATE ) {
				monthNode.setAttribute("label", MONTHLY_LABEL);
		}		            
 		else {
				monthNode.removeAttribute("label");
		}
		      
       // update complete
       var completeNode = selectedTreeItem.childNodes[0].childNodes[4];       
       if ( reminder.complete ) {
        	 completeNode.setAttribute("label", COMPLETED_LABEL);		
         }
 		else {
				completeNode.removeAttribute("label");
		}
		        
         // update remind until complete
         var remindUntilCompleteNode = selectedTreeItem.childNodes[0].childNodes[5];         
         if ( reminder.remindUntilCompleted != REMINDERFOX_REMIND_UNTIL_COMPLETE_NONE ) {
        	 remindUntilCompleteNode.setAttribute("label", COMPLETED_LABEL);		
         }
		else {
				remindUntilCompleteNode.removeAttribute("label");
		}         
}
  
  

  
  
  
function updateTodoInList( todo) {
	var modified = true;
	var treeChildren = document.getElementById("todoTreechildren");

	var selectedTreeItemIndex = document.getElementById("todoTree").currentIndex;
	var selectedTreeItem = treeChildren.childNodes[selectedTreeItemIndex];	
	var descriptionNode = selectedTreeItem.childNodes[0].childNodes[0];
	
	// update description
	descriptionNode.setAttribute( "label", todo.summary );
	
	 // update importance
	 if ( todo.priority == REMINDER_PRIORITY_IMPORTANT) { 
         // mark the description as important (red-highlighted)
        descriptionNode.setAttribute("properties", "important");
     }
     else {
        descriptionNode.removeAttribute("properties");
     }
   
   // update show in tooltip
   var completeNode = selectedTreeItem.childNodes[0].childNodes[1];       
   if ( todo.showInTooltip ) {
    	 completeNode.setAttribute("label", SHOW_IN_TOOLTIP_LABEL);		
     }
 	else {
		completeNode.removeAttribute("label");
	}    
		
	if ( modified ) {
		modifiedReminders();	
	}
	return modified;
		
}



function refreshCalendar() {
	calendarReminderArray = null;	
	oeDatePicker.redrawDays();
}


function toggleTodoShowInTooltip() {
	var currentTodo = getTodoForSelectedItem( );
	currentTodo.showInTooltip = !currentTodo.showInTooltip;
	var modified = updateTodoInList( currentTodo );
	modifiedReminders();
}

function toggleRemindUntilCompleted() {
	var currentReminder = getReminderForSelectedItem( );
	var cleanUpReminderInList = false;

	if ( currentReminder.remindUntilCompleted == REMINDERFOX_REMIND_UNTIL_COMPLETE_NONE ) {
		currentReminder.remindUntilCompleted = REMINDERFOX_REMIND_UNTIL_COMPLETE_TO_BE_MARKED;
	}
	else if ( currentReminder.remindUntilCompleted == REMINDERFOX_REMIND_UNTIL_COMPLETE_TO_BE_MARKED ) {
		currentReminder.remindUntilCompleted = REMINDERFOX_REMIND_UNTIL_COMPLETE_NONE;
	}
	else 	if ( currentReminder.remindUntilCompleted == REMINDERFOX_REMIND_UNTIL_COMPLETE_MARKED ) {
		currentReminder.remindUntilCompleted = REMINDERFOX_REMIND_UNTIL_COMPLETE_NONE;

		cleanUpReminderInList = true;
	}
		
	var modified = updateReminderInList( currentReminder );

	modifiedReminders();
	
	// if is was marked as RemindUntilCompleted and was showing in the list at a later date, we  need to remove this from the
	// UI list back to its original location
	if ( cleanUpReminderInList ) {
		removeListItemsForReminder(currentReminder);
		createListItem(currentReminder, false	);
	}
	
	refreshCalendar();
}


function toggleMarkAsComplete() {
	var currentReminder = getReminderForSelectedItem( );
	if ( currentReminder.complete ) { 
		currentReminder.complete = false;
	}
	else {
		if ( currentReminder.remindUntilCompleted == REMINDERFOX_REMIND_UNTIL_COMPLETE_TO_BE_MARKED ||
			  currentReminder.remindUntilCompleted == REMINDERFOX_REMIND_UNTIL_COMPLETE_MARKED ) {
			  
     		toggleRemindUntilCompleted();
     		return;     		
		}
		else {
			currentReminder.complete = true;
		}
	}
	var modified = updateReminderInList( currentReminder );

	modifiedReminders();
}