
// constants
const  reminderFox_TOOLTIP_LINE_LENGTH = 100;
const reminderFox_TOOLTIP_WITH_TODOS_LINE_LENGTH = 65;
const reminderFox_STATUS_TEXT_MAX_LENGTH = 40;
const reminderFox_ALERT_TEXT_MAX_LENGTH = 100;
const reminderFox_MONTHLY_WILDCARD = "*";


// Cache the last reminders
var reminderFox_lastDay = null;

// Launches the 'edit reminder' dialog, fills in Reminders list
function reminderFox_openAddRemindersDialog() {
   var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
   var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
   var topWindow = windowManagerInterface.getMostRecentWindow("window:reminderFoxEdit");
 
   if (topWindow) {
    	 topWindow.focus();
   }
   else {
		window.openDialog("chrome://reminderfox/content/addreminder-dialog.xul",
			"addreminder-dialog", "chrome,centerscreen,resizable", "reminders");
	}
}


function reminderFox_openOptionsDialog() {
   var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
   var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
   var topWindow = windowManagerInterface.getMostRecentWindow("window:reminderFoxOptions");
  
   if (topWindow) {
    	 topWindow.focus();
   }
   else {
		window.openDialog('chrome://reminderFox/content/options.xul', 'reminderFox-options', 'chrome');
	}
}


// Launches the 'edit reminder' dialog, fills in Reminders list
function reminderFox_openAddTodosDialog() {
   var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
   var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
   var topWindow = windowManagerInterface.getMostRecentWindow("window:reminderFoxEdit");
 
   if (topWindow) {
    	 topWindow.focus();
   }
   else {
	window.openDialog("chrome://reminderfox/content/addreminder-dialog.xul", 
		"addreminder-dialog", "chrome,centerscreen,resizable", "todos");
	}
}

function reminderFox_openDoubleClickDialog(event) {
    if (event.button == 0) {
		var defaultEditType = REMINDER_FOX_DEFAULT_EDIT_DEFAULT;			
		try {
			defaultEditType = reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_DEFAULT_EDIT);
		} catch(e) {
		}	
		
		if ( defaultEditType == "todos" )  {
			reminderFox_openAddTodosDialog();
		}
		else {
			reminderFox_openAddRemindersDialog();
		}
	}

}

// Launches the 'edit reminder' dialog, fills in Reminders list
function reminderFox_openAboutReminderFoxDialog() {
	window.openDialog("chrome://reminderfox/content/about.xul", "about", "chrome,centerscreen");
}


function reminderFox_processRecentReminders() {
	var changed = false;
	var NUM_OF_DAYS = 7;
	var i;
	var reminderEvents = 	reminderFox_getReminderEvents();
	var reminder;

	var currentDate = new Date();
	var year = currentDate.getFullYear();
	var month = currentDate.getMonth();
	var day = currentDate.getDate();

	var dayArray;
	if (reminderFox_isLeapYear(parseInt(year))) {
		dayArray = REMINDER_FOX_lDOMonth;
	}
	else {
		dayArray = REMINDER_FOX_DOMonth;
	}

	var dayOfYear=0;
	for ( i=0; i < month; i++) {
		dayOfYear += parseInt(dayArray[i]);
	}
	dayOfYear += parseInt(day);

	var removeArray = new Array();
	var removeArrayIndex = 0;
	for (  i = 0; i < reminderEvents.length; i++ ) {
		reminder = reminderEvents[i];
		if ( reminder.recurrence == REMINDER_RECURRENCE_MONTHLY_DATE ) {
			reminder = reminderFox_cloneReminderFoxEvent( reminder );
			reminder.date.setMonth(new Date().getMonth());
			reminder.date.setDate( reminderFox_getValidDateForMonth( new Date().getFullYear(), month, reminder.date.getDate() ) );		
		}

		var dayOfYearOfReminder= 0;
		for (var j=0; j < reminder.date.getMonth(); j++) {
			dayOfYearOfReminder += parseInt(dayArray[j]);
		}		
		dayOfYearOfReminder += parseInt(reminder.date.getDate());
				
		var diff = dayOfYear - dayOfYearOfReminder;
		var endDiff = -365 + NUM_OF_DAYS;

		// if the reminder is a one-time reminder that occurred in the last NUM_OF_DAYS, we'll
		// remove him
		if ((diff <= NUM_OF_DAYS && diff > 0) || diff < endDiff ) {		
			if ( reminder.recurrence == REMINDER_RECURRENCE_ONETIME ) {
					// mark this reminder for removal
					removeArray[removeArrayIndex] = i;
					removeArrayIndex++;
					changed = true;
			}
			
			// check for any RemindUntilComplete - if found, set it to be shown in today's list of reminders
			if ( reminder.remindUntilCompleted == REMINDERFOX_REMIND_UNTIL_COMPLETE_TO_BE_MARKED ) {
					reminder.remindUntilCompleted = REMINDERFOX_REMIND_UNTIL_COMPLETE_MARKED;
					reminderEvents[i].remindUntilCompleted = REMINDERFOX_REMIND_UNTIL_COMPLETE_MARKED;
				changed = true;
			}
			
			// if marked as complete, let's remove that mark so it shows up next year
			if ( reminder.complete ) {
				reminder.complete = false;
				changed = true;
			}
		}
		
	}
	
	// now let's remove our reminders
	for ( var k = removeArray.length - 1; k >= 0; k-- ) {
		reminderFox_removeElement( reminderEvents, removeArray[ k ] );		
	}
	
	return changed;
}



function reminderFox_createToolTip(todayRemindersArr) {
	var j;
	var currentNode;
	var todayDescription;
	var upcomingDescription;
	var spacer;
	var todoDescription;
	
	// we'll set the icon depending on if there's current, upcoming, 
	// or no reminders
 	var icon = document.getElementById('reminderFox-statusLabel');
 	icon.setAttribute("status", "noReminders");

 	var upcomingRemindersArr = reminderFox_getUpcomingReminders();
 	var todosArr = reminderFox_getVisibleTodos();

 	// clear old tooltips
	var todayBox = document.getElementById("todaysRemindersBox");
	if ( todayBox != null ) {
		var todaysremindersList = todayBox.childNodes;
		for ( j=0; j < todaysremindersList.length; j++) {
			currentNode = todaysremindersList[j];
			todayBox.removeChild(currentNode);
		}
	}
					
	var upcomingBox = document.getElementById("upcomingRemindersBox");
	if ( upcomingBox != null ) {
		var upcomingremindersList = upcomingBox.childNodes;
		for ( j=0; j < upcomingremindersList.length; j++) {
			currentNode = upcomingremindersList[j];
			upcomingBox.removeChild(currentNode);
		}
	}
	
	var todoBox = document.getElementById("todosBox");
	if ( todoBox != null ) {
		var todosList = todoBox.childNodes;
		for ( j=0; j < todosList.length; j++) {
			currentNode = todosList[j];
			todoBox.removeChild(currentNode);
		}
	}
		
	// remove old tooltip box
	var tooltipChildren = document.getElementById("tooltipChildren");
	todoDescription = document.getElementById( "todoDescription" );
	if ( todoDescription != null ) {
		tooltipChildren.removeChild( todoDescription );
	}
	var todosBox = document.getElementById( "todosBox" );
	if ( todosBox != null ) {
		tooltipChildren.removeChild( todosBox );		
	}
	
	// remove old reminders box
	var tooltipChildrenReminders = document.getElementById("tooltipChildrenReminders");
	todayDescription = document.getElementById( "todayRemindersDescription" );
	if ( todayDescription != null ) {
		tooltipChildrenReminders.removeChild( todayDescription );
	}
	
	todayBox = document.getElementById( "todaysRemindersBox" );
	if ( todayBox != null ) {
		tooltipChildrenReminders.removeChild( todayBox );		
	}
	
    spacer = document.getElementById( "reminderSpacer" );
	if ( spacer != null ) {
		tooltipChildrenReminders.removeChild( spacer );		
	}

	upcomingDescription = document.getElementById( "upcomingRemindersDescription" );
	if ( upcomingDescription != null ) {
		tooltipChildrenReminders.removeChild( upcomingDescription );
	}
	
	upcomingBox = document.getElementById( "upcomingRemindersBox" );
	if ( upcomingBox != null ) {
		tooltipChildrenReminders.removeChild( upcomingBox );		
	}	

	var showReminders = true;			
	try {
		showReminders = reminderFox_prefs.getBoolPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_SHOW_REMINDERS_IN_TOOLTIP);
	} catch(e) {
	}			

	var showTooltips = true;			
	try {
		showTooltips = reminderFox_prefs.getBoolPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_SHOW_TODOS_IN_TOOLTIP);
	} catch(e) {
	}			
 	
 	var tooltipWrapLength;
 	if ( todosArr.length > 0 && showReminders && showTooltips) {
 		tooltipWrapLength = reminderFox_TOOLTIP_WITH_TODOS_LINE_LENGTH;
 	}
 	else {
 		tooltipWrapLength = reminderFox_TOOLTIP_LINE_LENGTH;
 	}

 	if ( showReminders ) { 
 			todayDescription = document.createElement("description");
			todayDescription.setAttribute("id", "todayRemindersDescription");
			todayDescription.setAttribute("value", "Today's Reminders");
			todayDescription.setAttribute("style", "font-weight:bold;");
			tooltipChildrenReminders.appendChild(todayDescription);
	
			var todayVbox = document.createElement("vbox");
			todayVbox.setAttribute("id", "todaysRemindersBox");
			tooltipChildrenReminders.appendChild(todayVbox);
 				
			spacer =  	document.createElement("spacer");	
 			spacer.setAttribute("id", "reminderSpacer");		
 			spacer.setAttribute("height", "4px");		
			tooltipChildrenReminders.appendChild(spacer); 			
 		
 	 		upcomingDescription = document.createElement("description");
			upcomingDescription.setAttribute("id", "upcomingRemindersDescription");
			upcomingDescription.setAttribute("value", "Upcoming Reminders");
			upcomingDescription.setAttribute("style", "font-weight:bold;");
			tooltipChildrenReminders.appendChild(upcomingDescription);

			var upcomingVbox = document.createElement("vbox");
			upcomingVbox.setAttribute("id", "upcomingRemindersBox");
			tooltipChildrenReminders.appendChild(upcomingVbox);

	 		reminderFox_populateTodayToolTip(todayRemindersArr, tooltipWrapLength );
	 		reminderFox_populateUpcomingToolTip(upcomingRemindersArr, tooltipWrapLength);
	}
	
	if ( showTooltips && todosArr.length > 0 ) {
		todoDescription = document.createElement("description");
		todoDescription.setAttribute("id", "todoDescription");
		todoDescription.setAttribute("value", "ToDo's");
		todoDescription.setAttribute("style", "font-weight:bold;");
		tooltipChildren.appendChild(todoDescription);
		
		var todosVbox = document.createElement("vbox");
		todosVbox.setAttribute("id", "todosBox");
		tooltipChildren.appendChild(todosVbox);
		reminderFox_populateTodoToolTip(todosArr, (tooltipWrapLength - 13));	
	}
 		
	if ( todayRemindersArr.length > 0 ) {
		icon.setAttribute("status", "currentReminder");
	}
	else if ( upcomingRemindersArr.length > 0 ) {
		icon.setAttribute("status", "upcomingReminder");
	}	
}

function reminderFox_populateTodayToolTip(remindersArr, tooltipWrapLength) {
	var j;
	var remLabel;
	var lblparent = document.getElementById("todaysRemindersBox");
	if (remindersArr.length > 0 ) {

		var lineWrappedIndent = 5;	
		for(var i=0; i < remindersArr.length; i++) {		
			var startIndex = 0;
			var remainingString = remindersArr[i].summary;
			var afterFirst = false;
			while ( remainingString.length > tooltipWrapLength  ) {				
				// go back to last whitespace
				var lastIndex = remainingString.lastIndexOf( ' ', tooltipWrapLength );
				var forceLineBreak = false;
				if ( lastIndex == -1 ) {
					forceLineBreak = true;
			    	lastIndex = tooltipWrapLength + 1;
				}
				if ( lastIndex != -1 ) {
				    var curStr = remainingString.substring( 0, lastIndex );
				    // after the first line, for any extra lines we will indent some.This makes
				    // the entry look nice when wrapping to the following lines
				    if ( afterFirst )  {
				    	j = 0;
				    	while ( j < lineWrappedIndent ) {
				    		curStr = " " + curStr;
				    		j++;
				    	}
				    }	
				    	    
				    remLabel = document.createElement("description");
					remLabel.setAttribute("value", curStr);
					if ( remindersArr[i].priority == REMINDER_PRIORITY_IMPORTANT) { 
						remLabel.setAttribute("style", "color: red;");
					}
					else {
						remLabel.setAttribute("style", "color: black;");
					}
					lblparent.appendChild(remLabel);
					afterFirst = true;
				}
				
				if ( forceLineBreak ) {
					startIndex = lastIndex;
				}
				else {
					startIndex = lastIndex + 1;
				}
				remainingString = remainingString.substring( startIndex );		  										
			}
			if ( afterFirst )  {
			   	j = 0;
			   	while ( j < lineWrappedIndent ) {
			   		remainingString = " " + remainingString;
			   		j++;
			   	}
			}			

			remLabel = document.createElement("description");
			remLabel.setAttribute("value", remainingString);
			if ( remindersArr[i].priority == REMINDER_PRIORITY_IMPORTANT) { 
				remLabel.setAttribute("style", "color: red;");
			}
			else {
				remLabel.setAttribute("style", "color: black;");
			}
			lblparent.appendChild(remLabel);
		}
	} else {
		var noneLabel = document.createElement("description");
		noneLabel.setAttribute("value", "none");
		lblparent.appendChild(noneLabel);
	}
}


function reminderFox_populateUpcomingToolTip( remindersArr, tooltipWrapLength) {
	var j;
    var remLabel;
	var lblparent = document.getElementById("upcomingRemindersBox");
	if (remindersArr.length > 0 ) {
		for(var i=0; i < remindersArr.length; i++) {	
			var remainingString = reminderFox_getMonthAsText( remindersArr[i].date.getMonth()) + " " +  remindersArr[i].date.getDate() + ":  "  +  remindersArr[i].summary;
			

			var lineWrappedIndent = 5;	
				
			var startIndex = 0;
			var afterFirst = false;
			while ( remainingString.length > tooltipWrapLength ) {				
				// go back to last whitespace
				var lastIndex = remainingString.lastIndexOf( ' ', tooltipWrapLength );
				var forceLineBreak = false;
				if ( lastIndex == -1 ) {
					forceLineBreak = true;
			    	lastIndex = tooltipWrapLength + 1;
				}				
				if ( lastIndex != -1 ) {
				    var curStr = remainingString.substring( 0, lastIndex );
				    // after the first line, for any extra lines we will indent some. This makes
				    // the entry look nice when wrapping to the following lines
				    if ( afterFirst )  {
				    	j = 0;
				    	while ( j < lineWrappedIndent ) {
				    		curStr = " " + curStr;
				    		j++;
				    	}
				    }				    
				    
				    remLabel = document.createElement("description");
					remLabel.setAttribute("value", curStr);
					if ( remindersArr[i].priority == REMINDER_PRIORITY_IMPORTANT ) { 
						remLabel.setAttribute("style", "color: red;");
					}
					else {
						remLabel.setAttribute("style", "color: black;");
					}
					lblparent.appendChild(remLabel);
					afterFirst = true;
				}
								
				if ( forceLineBreak ) {
					startIndex = lastIndex;
				}
				else {
					startIndex = lastIndex + 1;
				}
				remainingString = remainingString.substring( startIndex );				    										
			}
			if ( afterFirst )  {
			   	j = 0;
			   	while ( j < lineWrappedIndent ) {
			   		remainingString = " " + remainingString;
			   		j++;
			   	}
			}
			
			remLabel = document.createElement("description");			
			remLabel.setAttribute("value", remainingString);
			if ( remindersArr[i].priority == REMINDER_PRIORITY_IMPORTANT ) {
				remLabel.setAttribute("style", "color: red;");
			}
			else {
				remLabel.setAttribute("style", "color: black;");
			}
			lblparent.appendChild(remLabel);
		}
	} else {
		var noneLabel = document.createElement("description");
		noneLabel.setAttribute("value", "none");
		lblparent.appendChild(noneLabel);
	}
}


function reminderFox_populateTodoToolTip( todosArr, tooltipWrapLength) {
	var j;
	var remLabel;
	var todosBox = document.getElementById("todosBox");
	
	if (todosArr.length > 0 ) {
		for(var i=0; i < todosArr.length; i++) {	
			var remainingString = todosArr[i].summary;
		
			var lineWrappedIndent = 5;
			
			var startIndex = 0;
			var afterFirst = false;
			while ( remainingString.length > tooltipWrapLength ) {				
				// go back to last whitespace
				var lastIndex = remainingString.lastIndexOf( ' ', tooltipWrapLength );
				var forceLineBreak = false;
				if ( lastIndex == -1 ) {
					forceLineBreak = true;
			    	lastIndex = tooltipWrapLength + 1;
				}					
				if ( lastIndex != -1 ) {
				    var curStr = remainingString.substring( 0, lastIndex );
				    // after the first line, for any extra lines we will indent some.  This makes
				    // the entry look nice when wrapping to the following lines
				    if ( afterFirst )  {
				    	j = 0;
				    	while ( j < lineWrappedIndent ) {
				    		curStr = " " + curStr;
				    		j++;
				    	}
				    }				    
				    
				    remLabel = document.createElement("description");
					remLabel.setAttribute("value", curStr);
					if ( todosArr[i].priority == REMINDER_PRIORITY_IMPORTANT ) {
						remLabel.setAttribute("style", "color: red;");
					}
					else {
						remLabel.setAttribute("style", "color: black;");
					}
					todosBox.appendChild(remLabel);
					afterFirst = true;
				}
								
				if ( forceLineBreak ) {
					startIndex = lastIndex;
				}
				else {
					startIndex = lastIndex + 1;
				}
				remainingString = remainingString.substring( startIndex );				    										
			}
			if ( afterFirst )  {
			   	j = 0;
			   	while ( j < lineWrappedIndent ) {
			   		remainingString = " " + remainingString;
			   		j++;
			   	}
			}
			
			remLabel = document.createElement("description");			
			remLabel.setAttribute("value", remainingString);
			if ( todosArr[i].priority == REMINDER_PRIORITY_IMPORTANT  ) {
				remLabel.setAttribute("style", "color: red;");
			}
			else {
				remLabel.setAttribute("style", "color: black;");
			}
			todosBox.appendChild(remLabel);
		}
	} else {
		var noneLabel = document.createElement("description");
		noneLabel.setAttribute("value", "none");
		todosBox.appendChild(noneLabel);
	}
}

function reminderFox_getUpcomingReminders() {
	var i, j;
	var reminder;  
	var val;
	var dayOfYearOfReminder;
	
	var reminderEvents = 	reminderFox_getReminderEvents();
	var upcomingReminders = new Array();

	var currentDate = new Date();
	var month = parseInt(currentDate.getMonth());
	var day = parseInt(currentDate.getDate());
	var year = parseInt(currentDate.getFullYear());

	var dayArray;
	if (reminderFox_isLeapYear(parseInt(year))) {
		dayArray = REMINDER_FOX_lDOMonth;
	}
	else {
		dayArray = REMINDER_FOX_DOMonth;
	}

	var dayOfYear=0;
	for ( i=0; i < month; i++) {
		dayOfYear += parseInt(dayArray[i]);
	}
	dayOfYear += parseInt(day);


	for( i = 0; i < reminderEvents.length; i++) {
		reminder = reminderEvents[i];
		dayOfYearOfReminder = 0;
		if ( reminder.recurrence == REMINDER_RECURRENCE_MONTHLY_DATE ) {
			reminder = reminderFox_cloneReminderFoxEvent( reminder );
			reminder.date.setMonth(month);
			reminder.date.setDate( reminderFox_getValidDateForMonth( new Date().getFullYear(), month, reminder.date.getDate() ) );		
			
			dayOfYearOfReminder= 0;
			for ( j=0; j < reminder.date.getMonth(); j++) {
				dayOfYearOfReminder += parseInt(dayArray[j]);
			}		
			dayOfYearOfReminder += parseInt(reminder.date.getDate());
			if ( dayOfYearOfReminder < dayOfYear ) {
				// this month's reminder has passed; let's use next month
				reminder.date.setMonth((month + 1) % 11);
				dayOfYearOfReminder= 0;
				for ( j=0; j <reminder.date.getMonth(); j++) {
					dayOfYearOfReminder += parseInt(dayArray[j]);
				}		
				dayOfYearOfReminder += parseInt(reminder.date.getDate());
			}						
		}		
		else {
			for ( j=0; j < reminder.date.getMonth(); j++) {
				dayOfYearOfReminder += parseInt(dayArray[j]);
			}		
			dayOfYearOfReminder += parseInt(reminder.date.getDate());
		}

		var upcomingReminderDays = parseInt(reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + UPCOMING_REMINDER_DAYS_PREF));

		var diff = dayOfYearOfReminder - dayOfYear;
		var endDiff = -365 + upcomingReminderDays;				
		if ((diff < upcomingReminderDays && diff > 0) || diff < endDiff ) {	
			// check for Completed status; if complete, skip it
			if ( reminder.complete ) {
				continue;
			}			
						
			// if it's a monthly reminder, it appears at the end of the list, so
			// we need to add it in the proper sorted order
			var index = reminderFox_getSortNewUpcomingReminderIndex( upcomingReminders, reminder );	
				
			// add 1 to their year value (can be replaced when reminders have years stored as part of their date)
			if ( reminder.date.getMonth() < currentDate.getMonth() ) {
				reminder  = reminderFox_processReminderDescription(reminder, currentDate.getFullYear() + 1);						
			}
			else {
				reminder = reminderFox_processReminderDescription(reminder, null);		
			}

			reminderFox_insertIntoArray( upcomingReminders, reminder, index );
		}
	}
		
	return upcomingReminders;
}


function reminderFox_getSortNewUpcomingReminderIndex( reminders, newreminder  ) {
	// we check if the Dates are from the next year; if so, put them at the end of the list by adding a year (12)
	// to their month value (can be replaced when reminders have years stored as part of their date)
	var currentDate = new Date();
	var newreminderMonth = newreminder.date.getMonth();
	if ( newreminderMonth < currentDate.getMonth() ) {
		newreminderMonth = newreminderMonth + 12;
	}

	// We go through each reminder and then compare that reminder's month with 
	// the reminder-to-be-added month, and if the new month is less, add there;
	// if the months are equal, then compare the days and add appropriately. 
	for( var i = 0; i < reminders.length; i++ ) {
		var currentReminder = reminders[i];
		var currentReminderMonth = currentReminder.date.getMonth();
		if ( currentReminderMonth < currentDate.getMonth()  ||  
						( currentReminderMonth ==  currentDate.getMonth() && newreminder.date.getDate() < currentDate.getDate() ) ) {
			currentReminderMonth = currentReminderMonth + 12;
		}		
		// add current reminder before reminders from later months
		if (    newreminderMonth< currentReminderMonth) { 
			return i;
		}
		// if they have the same month, then compare dates
		else if ( newreminderMonth  == currentReminderMonth ) {
			if ( newreminder.date.getDate() < currentReminder.date.getDate() ) {
				return i;
			}
		}
	}	

 	// if the reminder hasn't been added yet, then place it at the end
	// of the list
	return reminders.length;
}


function reminderFox_getTodaysReminders(  ) {
	var i; 
	var reminder, reminderArray, importantStatus;
	
	var reminderEvents = 	reminderFox_getReminderEvents();
	var todaysReminders = new Array();
	var todaysRemindersIndex = 0;

	var currentDate = new Date();
	var month = currentDate.getMonth();
	var day = currentDate.getDate();
	for ( i = 0; i < reminderEvents.length; i++ ) {
		reminder = reminderEvents[i];
		// if it's a monthly reminder, treat it as a reminder for current month
		if ( reminder.recurrence == REMINDER_RECURRENCE_MONTHLY_DATE ) {
			reminder = reminderFox_cloneReminderFoxEvent( reminder );
			reminder.date.setMonth(month);
			reminder.date.setDate( reminderFox_getValidDateForMonth( new Date().getFullYear(), month, reminder.date.getDate() ) );		
		}
		// if it's marked as RemindUntilComplete, this should be treated as Today's date
	 	if ( reminder.remindUntilCompleted == REMINDERFOX_REMIND_UNTIL_COMPLETE_MARKED ) {
	 		reminder = reminderFox_cloneReminderFoxEvent( reminder );
	 		var todaysDate = new Date();
			reminder.date = todaysDate;
	 	}		
		if (reminder.date.getMonth() == month && reminder.date.getDate() == day) {
			reminder = reminderFox_processReminderDescription(reminder, null);
			// check for Completed status; if complete, skip it
			if ( reminder.complete ) {
				continue;
			}			
			todaysReminders[todaysRemindersIndex] = reminder;
			todaysRemindersIndex++;			
		}
	}
	
	return todaysReminders;
}

function reminderFox_getVisibleTodos() {
	var todos = reminderFox_getReminderTodos();
	var visibleTodosArray = new Array();
	var	visibleTodosArrayIndex = 0;
	for(var  i = 0; i < todos.length; i++) {
		 if ( !todos[i].showInTooltip ) {
				continue;
		}			
		visibleTodosArray[visibleTodosArrayIndex] = todos[i];
		visibleTodosArrayIndex++;	
	}
	return visibleTodosArray;
}


function reminderFox_storeTimeOfLastUpdate() {
	// output the last time of update for debugging, but only if the
	// preference already exists
	try {		
		reminderFox_prefs.getCharPref(REMINDER_FOX_LAST_UPDATE);
		var currentDate = new Date();
		var hours = currentDate.getHours();
		var mins = currentDate.getMinutes();
		var secs = currentDate.getSeconds();
		
		var readableMins = mins;
		if ( mins < 10 ) {
			readableMins = "0" + readableMins;
		}
		var readableSecs = secs;
		if ( secs < 10 ) {
			readableSecs = "0" + readableSecs;
		}
		reminderFox_prefs.setCharPref(REMINDER_FOX_LAST_UPDATE, hours + ":" + readableMins + ":" + readableSecs);	
	} catch(e) {
		// pref doesn't exist.  Do nothing.
	}	
}


function reminderFox_moveBox() { 
    //check if box is ind the right location
    var toolbar = document.getElementById(reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_TOOLBAR));
    var box = document.getElementById("reminderFox-statusLabel");

    var position;
    try {
    	position = reminderFox_prefs.getIntPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_TOOLBAR_POSITION) ;
    }
    catch( e ) {
    	position = -1;
    }

    //if toolbar doesn't exist move to default location
    if (!toolbar) {
    	prefsBranch.setIntPref(REMINDER_FOX_TOOLBAR_POSITION, REMINDER_FOX_TOOLBAR_DEFAULT);
		prefsBranch.setIntPref(REMINDER_FOX_TOOLBAR_POSITION, REMINDER_FOX_TOOLBAR_POSITION_DEFAULT);      
     	return;
    }

    // bail if it is in the right place
    if (reminderFox_indexOf(toolbar,box, false) == position) {
	     return;
     }

    //remove us from parent
    box.parentNode.removeChild(box);

    reminderFox_insertAtIndex(toolbar, box, position);
  }
  

 function reminderFox_insertAtIndex(aParent, aChild, aIndex)  {
    var children = aParent.childNodes;
    if ((children.length == 0) || (aIndex >= children.length) || (aIndex < 0)) {
      aParent.appendChild(aChild);
     }
    else {
       aParent.insertBefore(aChild, children[aIndex]);
     }
  }

function reminderFox_storeTimeOfLastAlert() {
	// output the last time of update for debugging, but only if the
	// preference already exists
	try {		
		reminderFox_prefs.getCharPref(REMINDER_FOX_LAST_ALERT);
		var currentDate = new Date();
		var hours = currentDate.getHours();
		var mins = currentDate.getMinutes();
		var secs = currentDate.getSeconds();
		
		var readableMins = mins;
		if ( mins < 10 ) {
			readableMins = "0" + readableMins;
		}
		var readableSecs = secs;
		if ( secs < 10 ) {
			readableSecs = "0" + readableSecs;
		}
		reminderFox_prefs.setCharPref(REMINDER_FOX_LAST_ALERT, hours + ":" + readableMins + ":" + readableSecs);	
	} catch(e) {
		// pref doesn't exist.  Do nothing.
	}	
}


// this runs every hour and updates all of the windows with the latest reminders
function reminderFox_initializeReminderFox(clearReminders) {
	var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
	var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
	var windowEnumerator = windowManagerInterface.getEnumerator("navigator:browser");
	
	if (windowEnumerator.hasMoreElements()) {
		var oldestWindow = windowEnumerator.getNext();
		// if this is the very first (oldest) window opened, then let's process the reminders
		if ( window == oldestWindow ) {
			oldestWindow.reminderFox_loadReminderFox(false);	

			while (windowEnumerator.hasMoreElements()) {
				var currentWindow = windowEnumerator.getNext();		
				currentWindow.reminderFox_loadReminderFox(false);		
			}
			reminderFox_storeTimeOfLastUpdate();
			
			if ( clearReminders ) {
				// clear reminders from memory
				reminderFox_clearRemindersAndTodos();
			}
		}

		window.setTimeout(reminderFox_initializeReminderFoxHourly, 3600000);  // 10000 == 10 sec min
	}
}

// this runs every hour and updates all of the windows with the latest reminders
function reminderFox_initializeReminderFoxHourly() {
	reminderFox_initializeReminderFox(true);
}

function reminderFox_loadReminderFox(forceUpdate) {
	var reminderEvents = 	reminderFox_getReminderEvents();

	var changed = false;
	try { 
		changed = reminderFox_processRecentReminders();
	}
	catch ( e ) {
	}
	

	var currentDate = new Date();
	var day = currentDate.getDate();

	//only process if reminders have changed since last load, or it is a different
	// day (new reminders may need to be shown in statusbar/tooltips	
	if ( forceUpdate || reminderFox_lastDay  != day ) {
		reminderFox_lastDay  = day;
		var text = document.getElementById('reminderFox-statusLabel');
		if (text != null) {
			var reminderString = "";

			var remindersArray =reminderFox_getTodaysReminders();
			var important = false;
			for ( var i = 0; i < remindersArray.length; i++ ) {
				reminderString = reminderString + remindersArray[i].summary;
				if ( i < remindersArray.length - 1 ) {  
					reminderString = reminderString + ", ";
				}				
				if ( remindersArray[i].priority == REMINDER_PRIORITY_IMPORTANT ) {
					important = true;
				}
			}
			if (reminderString != "") { 
				// don't want text on status bar too long; truncate it after it gets big
				if ( reminderString.length > reminderFox_STATUS_TEXT_MAX_LENGTH ) {
					reminderString = reminderString.substring( 0, reminderFox_STATUS_TEXT_MAX_LENGTH ) + "...";
				}					

				if ( important ) {
					text.setAttribute("style", "color: red;");
				}
				else{
				    text.setAttribute("style", "color: black;");
				}

				var showStatusText = true;
				try {
					showStatusText = reminderFox_prefs.getBoolPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_SHOW_STATUS_TEXT);
				} catch(e) {
				}

				if ( showStatusText ) {
					text.setAttribute("class", "statusbarpanel-iconic-text");
					text.setAttribute("label", reminderString);					
				}
				else {
					// user doesn't want the reminder text - just use icon
					text.setAttribute("class", "statusbarpanel-iconic");
				}					
			} else {
				text.setAttribute("class", "statusbarpanel-iconic");
			}

			reminderFox_createToolTip(remindersArray);		
		}
	}
	
	if ( changed ) {	
		// write stuff out
		reminderFox_writeOutReminders();
		reminderFox_writeOutTodos();
	}
	
}


function reminderFox_activateOptionsContext(event) {	
	// dynamically handle the Show Alerts menu item 
	var alertType;
	try {
		alertType = reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_ENABLE_ALERT_PREF);
	} catch(e) {
		alertType = REMINDER_FOX_ENABLE_ALERT_PREF_ALL;
	}
			
	var showAlert = document.getElementById("options-contextmenu-showAlert");	
	
	// if the user has set it to no alerts, the option should be disabled
	if ( alertType == REMINDER_FOX_ENABLE_ALERT_PREF_NONE ) {
		showAlert.setAttribute("disabled", "true");
		showAlert.setAttribute("checked", "false");	    
	}
	// if alerts are suspended, the option is unchecked
	else if ( alertType.indexOf( REMINDER_FOX_SUSPEND_ALERT_PREF ) != -1 ) {
		showAlert.setAttribute("disabled", "false");
		showAlert.setAttribute("checked", "false");
		
	}
	// otherwise, in normal case the option is checked
	else {
		showAlert.setAttribute("disabled", "false");
		showAlert.setAttribute("checked", "true");
	}	

	var reminderFoxStatusbarPanel = document.getElementById("reminderFox-statusLabel");
	var optionsContextMenu = document.getElementById("options-contextmenu");
	var xCoord = event.clientX;
	var yCoord = event.clientY;
	document.popupNode = reminderFoxStatusbarPanel;
	optionsContextMenu.showPopup(reminderFoxStatusbarPanel, xCoord, yCoord, "bottomleft", "topleft");
}

function reminderFox_toggleShowAlert() {
	var alertType;
	try {
		alertType = reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_ENABLE_ALERT_PREF);
	} catch(e) {
		alertType = REMINDER_FOX_ENABLE_ALERT_PREF_ALL;
	}
	
	// if it's marked as suspended, change to original value
	var suspendedIndex = alertType.indexOf( REMINDER_FOX_SUSPEND_ALERT_PREF ) ;		
	if ( suspendedIndex != -1 ) {
		alertType = alertType.substring( suspendedIndex + REMINDER_FOX_SUSPEND_ALERT_PREF.length );
	}
	// otherwise, suspend the alert
	else {
		alertType = REMINDER_FOX_SUSPEND_ALERT_PREF + alertType;
	}
	
	reminderFox_prefs.setCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_ENABLE_ALERT_PREF, alertType);
}

function processKeyPressedOnReminderTree(event) {
	if (event.keyCode == 46)
		removeReminder();
}


function reminderFox_showAlertSlider() {
	var i;
	var remindersArray;
	var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
	var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
	var windowEnumerator = windowManagerInterface.getEnumerator("navigator:browser");
	
	if (windowEnumerator.hasMoreElements()) {
		var oldestWindow = windowEnumerator.getNext();
		// Only show the alert  for the oldest window (the first browser you opened).  
		// We don't want each new window to have its own timeout to respond to, because we only
		// need to show one alert for all browser windows
		if ( window == oldestWindow ) {
			var alerts = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
			var alertType;
			try {
				alertType = reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_ENABLE_ALERT_PREF);
			} catch(e) {
				alertType = REMINDER_FOX_ENABLE_ALERT_PREF_ALL;
			}
			
			if ( alertType == REMINDER_FOX_ENABLE_ALERT_PREF_NONE ) {
			    return;
			}
		
			var upcomingReminders = "";
			var todaysReminders = "";
			
			if ( alertType == REMINDER_FOX_ENABLE_ALERT_PREF_TODAY || alertType == REMINDER_FOX_ENABLE_ALERT_PREF_ALL ) {
				remindersArray = reminderFox_getTodaysReminders();
				for ( i = 0; i < remindersArray.length; i++ ) {
					todaysReminders = todaysReminders + remindersArray[i].summary;
					if ( i < remindersArray.length - 1 ) { 
						todaysReminders = todaysReminders + ", ";
					}
				}
			}
			
			if ( alertType ==REMINDER_FOX_ENABLE_ALERT_PREF_UPCOMING || alertType == REMINDER_FOX_ENABLE_ALERT_PREF_ALL ) {
				remindersArray = reminderFox_getUpcomingReminders();
				for ( i = 0; i < remindersArray.length; i++ ) {
					var upcomingStr = reminderFox_getMonthAsText(remindersArray[i].date.getMonth()) + " " + remindersArray[i].date.getDate() + ":  "  + remindersArray[i].summary;
					upcomingReminders = upcomingReminders + upcomingStr;
					if ( i < remindersArray.length - 1 ) { 
						upcomingReminders = upcomingReminders + ", ";
					}
				}
			}
			
			if ( todaysReminders != "" && upcomingReminders != "" ) {
				todaysReminders = todaysReminders + REMINDER_SEPARATOR;
			} 
			var totalReminders = todaysReminders + upcomingReminders;
		
			if (totalReminders != "") {
				// don't want text on alert slider too long; truncate it after it gets big
				if ( totalReminders > reminderFox_STATUS_TEXT_MAX_LENGTH ) {
					totalReminders = totalReminders.substring( 0, reminderFox_STATUS_TEXT_MAX_LENGTH ) + "...";
				}
			
				upcomingArray = totalReminders.split(REMINDER_SEPARATOR);
				totalReminders = "";
				for ( i =0; i < upcomingArray.length; i++) {
					totalReminders += upcomingArray[i];
					if (i != upcomingArray.length - 1)
						totalReminders += ", ";
				}
			
				// don't want text on status bar too long; truncate it after it gets big
				if ( totalReminders.length >reminderFox_ALERT_TEXT_MAX_LENGTH ) {
					totalReminders = totalReminders.substring( 0, reminderFox_ALERT_TEXT_MAX_LENGTH ) + "...";
				}
				alerts.showAlertNotification("chrome://reminderfox/skin/images/fox.png", "Upcoming Reminders", totalReminders, false, "", null);
				reminderFox_storeTimeOfLastAlert();
			}
		}
		
		// Reshow alert after ALERT_TIMEOUT minutes.
		var alert_timeout = parseInt(reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_ALERT_TIMEOUT_PREF));
		if ( alert_timeout > 0 ) {
			// convert from minutes to milliseconds	
			alert_timeout = alert_timeout * 60000; 
			window.setTimeout(reminderFox_showAlertSlider, alert_timeout);
		}
	}
}



function reminderFox_start() {
	reminderFox_loadDefaultPreferences();
	
	reminderFox_moveBox();
	
	// if this is the very first window opened, then let's set the timeout on it.
	reminderFox_initializeReminderFox(false);
	
	// Show the alert slider when the reminders are first initialized.
	// Only show if this is the oldest window (the first browser you opened).  That is, 
	// if you open a new window, we do not need to show the alert again.  
	reminderFox_showAlertSlider();

	// Initialize reminder fox for the browser window
	var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
	var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
	var windowEnumerator = windowManagerInterface.getEnumerator("navigator:browser");
	if (windowEnumerator.hasMoreElements()) {
		var oldestWindow = windowEnumerator.getNext();
		if ( window != oldestWindow ) {
			//  it's just a new window, we'll load it (tooltips, status bar)
			reminderFox_loadReminderFox(false);	
		}
	}

	// clear reminders from memory
	reminderFox_clearRemindersAndTodos();
}

/*
function reminderFox_stop() {
	var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
	var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
	var windowEnumerator = windowManagerInterface.getEnumerator("navigator:browser");
	if (windowEnumerator.hasMoreElements()) {
		var oldestWindow = windowEnumerator.getNext();
		// if this is the very first window opened and it's closed, let's set the timeouts on the next oldest window
		if ( window == oldestWindow ) {
			var nextOldestWindow = windowEnumerator.getNext();
			if ( nextOldestWindow != null ) {
					nextOldestWindow.setTimeout(reminderFox_initializeReminderFox, 3600000);  // 10000 == 1 min
					var alert_timeout = parseInt(reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_ALERT_TIMEOUT_PREF));
					// convert from minutes to milliseconds	
					alert_timeout = alert_timeout * 60000; 
					nextOldestWindow.setTimeout(reminderFox_showAlertSlider, alert_timeout);
			}
		}
	}
}
*/


