// constants
const REMINDER_PROPERTY_DELIMITER = "`";
const REMINDER_SEPARATOR = "~";
const REMINDER_FOX_OCCURRENCE_ONCE = "1";
const REMINDER_TO_BE_COMPLETED_DELIMITER = "@";
const REMINDER_FOX_MONTHLY_WILDCARD_CHAR = '*';
const REMINDER_FOX_MONTHLY_WILDCARD_STR = "*";
const REMINDER_IMPORTANT_FLAG = "!";

const REMINDER_PRIORITY_NORMAL = 0;
const REMINDER_PRIORITY_IMPORTANT = 1;

const REMINDER_RECURRENCE_YEARLY = 0;
const REMINDER_RECURRENCE_MONTHLY_DATE = 1;
const REMINDER_RECURRENCE_ONETIME = 2;

const REMINDERFOX_REMIND_UNTIL_COMPLETE_NONE = 0;
const REMINDERFOX_REMIND_UNTIL_COMPLETE_TO_BE_MARKED = 1;
const REMINDERFOX_REMIND_UNTIL_COMPLETE_MARKED = 2;

const REMINDER_FOX_DOMonth = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];// # of days in Non-Leap year Month 
const REMINDER_FOX_lDOMonth = [31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]; // # of days in Leap year Month 

// configurable preferences   
const REMINDER_FOX_ENABLE_ALERT_PREF = "enableAlertFor"; 
const REMINDER_FOX_ENABLE_ALERT_PREF_NONE = "none"; 
const REMINDER_FOX_ENABLE_ALERT_PREF_TODAY = "today"; 
const REMINDER_FOX_ENABLE_ALERT_PREF_UPCOMING = "upcoming"; 
const REMINDER_FOX_ENABLE_ALERT_PREF_ALL = "all"; 
const REMINDER_FOX_SUSPEND_ALERT_PREF = "SUSPENDED:";

const REMINDER_FOX_ALERT_TIMEOUT_PREF="alertTimeout";
const REMINDER_FOX_ALERT_TIMEOUT_DEFAULT = 120; // number of minutes between alerts

const UPCOMING_REMINDER_DAYS_PREF="upcomingReminderDays";
const UPCOMING_REMINDER_DAYS_DEFAULT = 15;

const REMINDER_FOX_SHOW_STATUS_TEXT="showReminderStatusText";
const REMINDER_FOX_SHOW_STATUS_TEXT_DEFAULT=true;

const REMINDER_FOX_SHOW_REMINDERS_IN_TOOLTIP="showRemindersInTooltip";
const REMINDER_FOX_SHOW_REMINDERS_IN_TOOLTIP_DEFAULT=true;

const REMINDER_FOX_SHOW_TODOS_IN_TOOLTIP="showTodosInTooltip";
const REMINDER_FOX_SHOW_TODOS_IN_TOOLTIP_DEFAULT=true;

const REMINDER_FOX_TOOLBAR="toolbar";
const REMINDER_FOX_TOOLBAR_DEFAULT="status-bar";

const REMINDER_FOX_DEFAULT_EDIT="defaultEdit";
const REMINDER_FOX_DEFAULT_EDIT_DEFAULT="reminders";

const REMINDER_FOX_TOOLBAR_POSITION = "toolbarPosition";
const REMINDER_FOX_TOOLBAR_POSITION_DEFAULT = -1;

const REMINDER_FOX_MIGRATED_PREF="migrated";
const REMINDER_FOX_MIGRATED_PREF_VERSION="0.5.8";

const REMINDER_FOX_PREF = "reminderFox";
const REMINDER_FOX_REMINDERS_PREF = "reminders";
const REMINDER_FOX_REMINDERS = REMINDER_FOX_PREF + "." + REMINDER_FOX_REMINDERS_PREF;

const REMINDER_FOX_TODOS_PREF = "todos";
const REMINDER_FOX_TODOS = REMINDER_FOX_PREF + "." + REMINDER_FOX_TODOS_PREF;

const REMINDER_FOX_DEBUG = "debug";
const REMINDER_FOX_LAST_UPDATE = REMINDER_FOX_PREF + "." +REMINDER_FOX_DEBUG + "." + "lastUpdate";
const REMINDER_FOX_LAST_ALERT = REMINDER_FOX_PREF + "." +REMINDER_FOX_DEBUG + "." + "lastAlert";

const reminderFox_prefs = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefBranch);

const reminderFox_nsISupportsString = Components.interfaces.nsISupportsString;

var reminderFoxEvents;
var reminderFoxTodos;

 var g_reminderFox_uniqueEventId = 0;
 var g_reminderFox_uniqueTodoId = 0;
 
/* get unicode string value from preference store */
function reminderFox_getUnicodePref(prefName) {
    return reminderFox_prefs.getComplexValue(prefName, reminderFox_nsISupportsString).data;
}

/* set unicode string value */
function reminderFox_setUnicodePref(prefName,prefValue) {
    var sString = Components.classes["@mozilla.org/supports-string;1"].createInstance(reminderFox_nsISupportsString);
    sString.data = prefValue;
    reminderFox_prefs.setComplexValue(prefName,reminderFox_nsISupportsString,sString);
}

// Object representing reminder events
function ReminderFoxEvent( reminderDate, reminderSummary ) {
	this.id = g_reminderFox_uniqueEventId++;    // a unique identifier for this reminder (used at edit time)
    this.date = reminderDate;    // date object for reminder (using current year)
    this.summary = reminderSummary;  // text of reminder
    this.recurrence = REMINDER_RECURRENCE_YEARLY;  
    this.priority = REMINDER_PRIORITY_NORMAL;
    this.remindUntilCompleted = REMINDERFOX_REMIND_UNTIL_COMPLETE_NONE;
    this.originalDate = this.date;
    this.allDayEvent = true;
    this.alarm = null;
    this.complete = false;
    this.categories = null;
}

// Object representing ToDos
function ReminderFoxTodo( todoSummary ) {
	this.id = g_reminderFox_uniqueTodoId++;    // a unique identifier for this todo (used at edit time)
    this.summary = todoSummary;
    this.priority = REMINDER_PRIORITY_NORMAL;
    this.dateStamp = null;
    this.categories = null;
    this.showInTooltip = false;
    this.complete = false;   // has this ToDo been marked as completed?
}

/*
 * make a clone object of an event - useful if you want to temporarily modify some properties of an event, 
 * but don't want to change the actual model 
 */
function reminderFox_cloneReminderFoxEvent( reminderFoxEvent ) {
	var newReminderFoxEvent = new ReminderFoxEvent( reminderFoxEvent.date, reminderFoxEvent.summary );
	newReminderFoxEvent.id = reminderFoxEvent.id;
	newReminderFoxEvent.recurrence = reminderFoxEvent.recurrence;
	newReminderFoxEvent.priority = reminderFoxEvent.priority;
	newReminderFoxEvent.remindUntilCompleted = reminderFoxEvent.remindUntilCompleted;
	newReminderFoxEvent.originalDate = reminderFoxEvent.originalDate;
	newReminderFoxEvent.complete = reminderFoxEvent.complete;	
	newReminderFoxEvent.categories = reminderFoxEvent.categories;
	return newReminderFoxEvent;
}	

/*
 * make a clone object of an event - useful if you want to temporarily modify some properties of an event, 
 * but don't want to change the actual model 
 */
function reminderFox_cloneReminderFoxTodo( reminderFoxTodo ) {
	var newReminderFoxTodo = new ReminderFoxTodo(  reminderFoxTodo.summary );
	newReminderFoxTodo.id = reminderFoxTodo.id;
	newReminderFoxTodo.priority = reminderFoxTodo.priority;
	newReminderFoxTodo.showInTooltip = reminderFoxTodo.showInTooltip;
	newReminderFoxTodo.complete = reminderFoxTodo.complete;
	newReminderFoxTodo.categories = reminderFoxTodo.categories;
	return newReminderFoxTodo;
}	

function reminderFox_isLeapYear(year) { 
	return (((year % 4 == 0) && (year % 100 != 0)) || (year % 400 == 0)) ? true : false;
}



function reminderFox_getMonthAsInteger(monthTextValue) {
	var prefMonth;
	switch (monthTextValue) {
		case "January":
		prefMonth = 0;
		break;

		case "February":
		prefMonth = 1;
		break;

		case "March":
		prefMonth = 2;
		break;

		case "April":
		prefMonth = 3;
		break;

		case "May":
		prefMonth = 4;
		break;

		case "June":
		prefMonth = 5;
		break;

		case "July":
		prefMonth = 6;
		break;

		case "August":
		prefMonth = 7;
		break;

		case "September":
		prefMonth = 8;
		break;

		case "October":
		prefMonth = 9;
		break;

		case "November":
		prefMonth = 10;
		break;

		case "December":
		prefMonth = 11;
		break;
		
		case "Monthly":
		prefMonth = REMINDER_FOX_MONTHLY_WILDCARD_CHAR;
		break;
	}
	return prefMonth;

}


function reminderFox_getMonthAsText(integerValue) {
	var monthAsText;
	
	integerValue = integerValue + "";  // convert to string
	
	switch (integerValue) {
		case "0":
		monthAsText = "January";
		break;

		case "1":
		monthAsText = "February";
		break;

		case "2":
		monthAsText = "March";
		break;

		case "3":
		monthAsText = "April";
		break;

		case "4":
		monthAsText = "May";
		break;

		case "5":
		monthAsText = "June";
		break;

		case "6":
		monthAsText = "July";
		break;

		case "7":
		monthAsText = "August";
		break;

		case "8":
		monthAsText = "September";
		break;

		case "9":
		monthAsText = "October";
		break;

		case "10":
		monthAsText = "November";
		break;

		case "11":
		monthAsText = "December";
		break;
	}
	return monthAsText;
}


function reminderFox_loadDefaultPreferences() {
	var prefsBranch = Components.classes["@mozilla.org/preferences-service;1"]. 
				getService(Components.interfaces.nsIPrefService). 
				getBranch(REMINDER_FOX_PREF + ".");

	// If the reminders preferences do not exist create them with default values
	try {
		reminderFox_prefs.getCharPref(REMINDER_FOX_REMINDERS);
	} catch(e) {
		prefsBranch.setCharPref(REMINDER_FOX_REMINDERS_PREF, "");
	}	
	
	try {
		reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_ENABLE_ALERT_PREF);
	} catch(e) {
		prefsBranch.setCharPref(REMINDER_FOX_ENABLE_ALERT_PREF, REMINDER_FOX_ENABLE_ALERT_PREF_ALL);
	}
	
	try {
		reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_ALERT_TIMEOUT_PREF);
	} catch(e) {
		prefsBranch.setCharPref(REMINDER_FOX_ALERT_TIMEOUT_PREF, REMINDER_FOX_ALERT_TIMEOUT_DEFAULT);
	}
	
	try {
		reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + UPCOMING_REMINDER_DAYS_PREF);
	} catch(e) {
		prefsBranch.setCharPref(UPCOMING_REMINDER_DAYS_PREF, UPCOMING_REMINDER_DAYS_DEFAULT);
	}
	
	try {
		reminderFox_prefs.getCharPref(REMINDER_FOX_TODOS);
	} catch(e) {
		prefsBranch.setCharPref(REMINDER_FOX_TODOS_PREF, "");
	}	
	
	try {
		reminderFox_prefs.getBoolPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_SHOW_STATUS_TEXT);
	} catch(e) {
		prefsBranch.setBoolPref(REMINDER_FOX_SHOW_STATUS_TEXT, REMINDER_FOX_SHOW_STATUS_TEXT_DEFAULT);
	}	
	
	try {
		reminderFox_prefs.getBoolPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_SHOW_REMINDERS_IN_TOOLTIP);
	} catch(e) {
		prefsBranch.setBoolPref(REMINDER_FOX_SHOW_REMINDERS_IN_TOOLTIP, REMINDER_FOX_SHOW_REMINDERS_IN_TOOLTIP_DEFAULT);
	}	

	try {
		reminderFox_prefs.getBoolPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_SHOW_TODOS_IN_TOOLTIP);
	} catch(e) {
		prefsBranch.setBoolPref(REMINDER_FOX_SHOW_TODOS_IN_TOOLTIP, REMINDER_FOX_SHOW_TODOS_IN_TOOLTIP_DEFAULT);
	}					
		
	try {
		reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_TOOLBAR);
	} catch(e) {
		prefsBranch.setCharPref(REMINDER_FOX_TOOLBAR, REMINDER_FOX_TOOLBAR_DEFAULT);
	}		
		
	try {
		reminderFox_prefs.getIntPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_TOOLBAR_POSITION);
	} catch(e) {
		prefsBranch.setIntPref(REMINDER_FOX_TOOLBAR_POSITION, REMINDER_FOX_TOOLBAR_POSITION_DEFAULT);
		
		try {
			// set the default location to the current location as of this moment
		    var reminderFox_toolbar = document.getElementById(reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_TOOLBAR));
		    var box = document.getElementById("reminderFox-statusLabel");
		    if (reminderFox_toolbar) {
		    	var currentIndex = reminderFox_indexOf(reminderFox_toolbar, box, true);
		    	prefsBranch.setIntPref(REMINDER_FOX_TOOLBAR_POSITION, currentIndex);	    	
		    }
		}
		catch (e) {
		}
	}	
	
	try {
		reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_DEFAULT_EDIT);
	} catch(e) {
		prefsBranch.setCharPref(REMINDER_FOX_DEFAULT_EDIT, REMINDER_FOX_DEFAULT_EDIT_DEFAULT);
	}		

	try {
		var oldVersionNumber = reminderFox_prefs.getCharPref(REMINDER_FOX_PREF + "." + REMINDER_FOX_MIGRATED_PREF);
		if ( oldVersionNumber != REMINDER_FOX_MIGRATED_PREF_VERSION ) {
			// this is an older version; we need to handle migration code	
			reminderFox_handleMigration( oldVersionNumber );				
		}		
	} catch(e) {
		// this is an older version; we need to handle migration code		
		reminderFox_handleMigration("0.0");
	}
}

function reminderFox_getSortedIndexOfNewReminder(  newreminder, addUniqueOnly  ) {
	var reminderFoxEvents = reminderFox_getReminderEvents();	
	
	var month = newreminder.date.getMonth();
	if ( newreminder.recurrence == REMINDER_RECURRENCE_MONTHLY_DATE ) {
		month = 12;
	}	

	// We go through each reminder and then compare that reminder's month with 
	// the reminder-to-be-added month, and if the new month is less, add there;
	// if the months are equal, then compare the days and add appropriately. 
	for ( var i = 0; i < reminderFoxEvents.length; i++ ) {
		var reminder = reminderFoxEvents[i];
		
		// put the multi-recurring reminders at the end after the regulation reminders
		if ( newreminder.recurrence == REMINDER_RECURRENCE_MONTHLY_DATE && 
			reminder.recurrence != REMINDER_RECURRENCE_MONTHLY_DATE ) {
			continue;
		}	
		
		// add current reminder before reminders from later months
		if (   month < reminder.date.getMonth()) { 
			return i;
		}
		// if they have the same month, then compare dates
		else if ( month == reminder.date.getMonth() ) {
			if ( newreminder.date.getDate()  <  reminder.date.getDate() ) {
				return i;
			}
			else if (  newreminder.date.getDate() == reminder.date.getDate() ) {					
				if ( addUniqueOnly && newreminder.summary == reminder.summary ) {
					// this reminder is already in the list; just ignore it
					return -1;
				}
				else {
					// if this is an Important reminder, add it before other reminders of the same day
					if ( newreminder.priority == REMINDER_PRIORITY_IMPORTANT  ) {
						return i;
					}
				}
			}
		}  // monthly			
	} // for
	
	return reminderFoxEvents.length;
}	

  function reminderFox_indexOf(aParent, aChild, overrideLast)  { 
    // -1 if it does not exist
    var children = aParent.childNodes;
    if ( children == null ) {
    	return null;
    }
    for (var x=0; x<children.length; x++) {
	    if (children[x] == aChild) {
			if ( overrideLast ) {
				return x;
			}
			else {
				return (x == children.length - 1) ? -1 : x;
			}
	    }
    }
    return null;
  }
  




function reminderFox_getSortedIndexOfNewTodo(  newTodo ) {
	var reminderFoxTodos = reminderFox_getReminderTodos();	

	// if current todo is important, add to start of list
    if (newTodo.priority == REMINDER_PRIORITY_IMPORTANT  ) {
		// insert at beginning
		return 0;
    }
    else {
    	// run through until find one that's not important, and add new one before it
		 for( var i = 0; i < reminderFoxTodos.length; i++) {
	        if ( !reminderFoxTodos[i].priority == REMINDER_PRIORITY_IMPORTANT  ) {
	        	if ( newTodo.showInTooltip ) {
					return i;
	        	}
	        	else {
	        		if ( !reminderFoxTodos[i].showInTooltip ) {
						return i;
	        		}
	        	}	        
	        }
		}
    }
	
	// if the reminder hasn't been added yet, then place it at the end
	return reminderFoxTodos.length;			
}
 
 function reminderFox_getValidDateForMonth( year, month, date ) {
	var lastValidDate = date;	
	var lastDayOfMonth = oeDatePicker.getLastDayOfMonth( year, month );
	if ( date > lastDayOfMonth ) {
		lastValidDate = lastDayOfMonth;
	}
	return lastValidDate;
}
 
 /*
  *DEPRECATED - for migration only 
 */
 function reminderFox_sortNewReminder( reminders, newreminder, monthInt, dayInt, addUniqueOnly  ) {
	var leftSideOfReminders, rightSideOfReminders;
	
	// We go through each reminder and then compare that reminder's month with 
	// the reminder-to-be-added month, and if the new month is less, add there;
	// if the months are equal, then compare the days and add appropriately. 
	if (reminders != "") {
	 	var index = 0;
		var endIndex = reminders.indexOf( REMINDER_SEPARATOR );
		if ( endIndex == -1 ) {
			endIndex = reminders.length ;
		}
		var added = false;
		while ( endIndex != -1 && !added ) {
			var currentReminder;
			if ( endIndex == reminders.length - 1 ) {
				currentReminder = reminders.substring( index, endIndex + 1 );
			}
			else {
				currentReminder = reminders.substring( index, endIndex );
			}
			var reminderArray = currentReminder.split(REMINDER_PROPERTY_DELIMITER);

			var myMonth;
			var reminderMon;
			// treat Monthly reminders as coming at the end of the list
			if ( monthInt == REMINDER_FOX_MONTHLY_WILDCARD_CHAR ) {
				myMonth = 12;
			}
			else {
			    myMonth = parseInt( monthInt );
			}
			if ( reminderArray[0] == REMINDER_FOX_MONTHLY_WILDCARD_CHAR ) { 
				reminderMon = 12;
			}
			else {
				reminderMon = parseInt(reminderArray[0]);
			}
		
			// add current reminder before reminders from later months
			if (  myMonth < reminderMon) { 
				leftSideOfReminders = reminders.substring(0, index );						
				rightSideOfReminders = reminders.substring(index ,  reminders.length);					
				reminders = leftSideOfReminders + newreminder + REMINDER_SEPARATOR + rightSideOfReminders;
				added = true;
			}
			// if they have the same month, then compare dates
			else if ( reminderMon == myMonth ) {
				var myDay = parseInt(dayInt);
				var reminderDay = parseInt(reminderArray[1]);
				if ( myDay < reminderDay ) {
					leftSideOfReminders = reminders.substring(0, index );							
					rightSideOfReminders = reminders.substring(index ,  reminders.length);					
					reminders = leftSideOfReminders + newreminder + REMINDER_SEPARATOR + rightSideOfReminders;
					added = true;
				}
				else if ( myDay == reminderDay ) {
					var newReminderArray = newreminder.split(REMINDER_PROPERTY_DELIMITER);
					if ( addUniqueOnly && newReminderArray[2] == reminderArray[2] ) {
						// this reminder is already in the list; just ignore it
						added = true;
					}
					else {
						// if this is an Important reminder, add it before other reminders of the same day
						var indexImportant = newreminder.indexOf( REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG );
						if ( indexImportant != -1 ) {
							leftSideOfReminders = reminders.substring(0, index );							
							rightSideOfReminders = reminders.substring(index ,  reminders.length);					
							reminders = leftSideOfReminders + newreminder + REMINDER_SEPARATOR + rightSideOfReminders;
							added = true;
						}
					}
				}
			}
			index = endIndex + 1;
			endIndex = reminders.indexOf( REMINDER_SEPARATOR, index );		
			if ( index != reminders.length && endIndex == -1 ) {
				endIndex = reminders.length - 1;
			}
		}
		
		// if the reminder hasn't been added yet, then place it at the end
		// of the list
		if ( !added ) {
			reminders += REMINDER_SEPARATOR + newreminder;
		}	
	}	
	else {
		reminders += newreminder;
	}	
	return reminders;
}
 
function reminderFox_handleMigration(oldVersion) {
		// this is an older version; we need to handle migration code		
		var prefsBranch = Components.classes["@mozilla.org/preferences-service;1"]. 
				getService(Components.interfaces.nsIPrefService). 
				getBranch(REMINDER_FOX_PREF + ".");
		
		var reminders = "";
		try {
			reminders = reminderFox_getUnicodePref(REMINDER_FOX_REMINDERS);
		}
		catch (e) {
		}
		
		if ( reminders != "" ) {
			if ( oldVersion < "0.3" ) {
				// replace old delimeters
				reminders = reminders.replace( /\$/g, REMINDER_PROPERTY_DELIMITER );
				reminders = reminders.replace( /,/g, REMINDER_SEPARATOR );
			}
			
			// now sort all of the reminders
			var newReminderStr = "";			
			if ( reminders.lastIndexOf( REMINDER_SEPARATOR ) != reminders.length - 1 ) {
				reminders = reminders + REMINDER_SEPARATOR;
			}
			
			var remindersArray = reminders.split(REMINDER_SEPARATOR);

			for(var i = 0; i < remindersArray.length; i++) {
				var newreminder = remindersArray[i];
				var reminderArray = newreminder.split(REMINDER_PROPERTY_DELIMITER);			
				newReminderStr = reminderFox_sortNewReminder( newReminderStr, newreminder, reminderArray[0], reminderArray[1] , false );				
			}
			
			reminders = newReminderStr;
			if ( reminders.lastIndexOf( REMINDER_SEPARATOR ) == reminders.length - 1 ) {
				reminders = reminders.substring( 0, reminders.length - 1);
			}
			reminderFox_setUnicodePref(REMINDER_FOX_REMINDERS, reminders);
			
		}
		prefsBranch.setCharPref(REMINDER_FOX_MIGRATED_PREF, REMINDER_FOX_MIGRATED_PREF_VERSION);		
}





function reminderFox_writeOutReminders() {
	var reminderFoxEvents = reminderFox_getReminderEvents();
	var reminder;
	var reminderString = "";
	
	for ( var i = 0; i < reminderFoxEvents.length; i++ ) {
		reminder = reminderFoxEvents[i];
		var month = reminder.date.getMonth() + "";

		if ( reminder.recurrence == REMINDER_RECURRENCE_MONTHLY_DATE ) {
			month = REMINDER_FOX_MONTHLY_WILDCARD_STR;
		}
		var newReminder = month + REMINDER_PROPERTY_DELIMITER + reminder.date.getDate() + 
			REMINDER_PROPERTY_DELIMITER + reminder.summary;
		
		if (reminder.priority == REMINDER_PRIORITY_IMPORTANT  ) {
			newReminder = newReminder + REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG;
		}
		if ( reminder.complete ) {
			newReminder = newReminder + REMINDER_PROPERTY_DELIMITER + REMINDER_TO_BE_COMPLETED_DELIMITER;
		}
		if ( reminder.remindUntilCompleted == REMINDERFOX_REMIND_UNTIL_COMPLETE_TO_BE_MARKED) {
			newReminder = newReminder + REMINDER_PROPERTY_DELIMITER + REMINDER_TO_BE_COMPLETED_DELIMITER +
				reminder.originalDate.getMonth() +"-" +  reminder.originalDate.getDate();
		}
		else if ( reminder.remindUntilCompleted == REMINDERFOX_REMIND_UNTIL_COMPLETE_MARKED ) {
			newReminder = newReminder + REMINDER_PROPERTY_DELIMITER + REMINDER_TO_BE_COMPLETED_DELIMITER +REMINDER_TO_BE_COMPLETED_DELIMITER+
				reminder.originalDate.getMonth() +"-" +  reminder.originalDate.getDate();
		}
		if ( reminder.recurrence == REMINDER_RECURRENCE_ONETIME ) {
			newReminder = newReminder + REMINDER_PROPERTY_DELIMITER + REMINDER_FOX_OCCURRENCE_ONCE;
		}		
		
		if ( reminderString != "" ) {
			reminderString = reminderString + REMINDER_SEPARATOR;
		}
		reminderString = reminderString + newReminder;	
	}
	reminderFox_setUnicodePref( REMINDER_FOX_REMINDERS, reminderString );
}
	

// index is the number of the element you wish to remove
// from the array: reminderFoxEvents	
function reminderFox_removeElement(reminderFoxEvents, index) {

    for (index = index; index<reminderFoxEvents.length;index++) {
        // assigns the value of elementnr+1 to elementnr, so you move all items by 1
        reminderFoxEvents[index] = reminderFoxEvents[index + 1];
    }
    reminderFoxEvents.length=reminderFoxEvents.length-1;
    
}

// position is the number where you want to add the new element (reminder) into 
// the array: reminderFoxEvents	
function reminderFox_insertIntoArray(reminderFoxEvents, reminder, position) {
	var origLength = reminderFoxEvents.length;
	if ( position > origLength ) {
		position = origLength;
	}
	
	reminderFoxEvents.length = reminderFoxEvents.length + 1;

	for ( var i = origLength; i >=0; i-- ) {
		if ( i == position ) {
			reminderFoxEvents[i] = reminder;
			break;
		}
		else if ( i > position ) {
			reminderFoxEvents[i] = reminderFoxEvents[i-1];
		}
		else if ( i < position ) {
			break;
		}
	}
}
	
	
	function reminderFox_writeOutTodos() {
		var reminderFoxTodos = reminderFox_getReminderTodos();
		var todo;
		
		var todosString = "";
		for ( var i = 0; i < reminderFoxTodos.length; i++ ) {
			todo = reminderFoxTodos[i];

			var newTodo = todo.summary;
			
			if (todo.priority == REMINDER_PRIORITY_IMPORTANT  ) {
				newTodo = newTodo + REMINDER_PROPERTY_DELIMITER + REMINDER_IMPORTANT_FLAG;
			}
			if ( todo.showInTooltip ) {
				newTodo = newTodo + REMINDER_PROPERTY_DELIMITER + REMINDER_TO_BE_COMPLETED_DELIMITER;
			}
			
			if ( todosString != "" ) {
				todosString = todosString + REMINDER_SEPARATOR;
			}
			todosString = todosString + newTodo;	

		}
		reminderFox_setUnicodePref( REMINDER_FOX_TODOS, todosString );
	}
	

	function reminderFox_readInRemindersAndTodos() {
		reminderFoxEvents = new Array();
		reminderFoxTodos = new Array();
		var i, j;
		
		var reminders = "";
		try {
			reminders = reminderFox_getUnicodePref(REMINDER_FOX_REMINDERS);
		}
		catch (e) {
		}
		if ( reminders != "" ) {
			var remindersArray = reminders.split(REMINDER_SEPARATOR);
			for( i = 0; i < remindersArray.length; i++) {
	
					var newreminder = remindersArray[i];
					var reminderArray = newreminder.split(REMINDER_PROPERTY_DELIMITER);			
					var month = reminderArray[0];
					var day = reminderArray[1];
					
					var date = new Date();
					if ( reminderArray[0] == REMINDER_FOX_MONTHLY_WILDCARD_CHAR ) {
						month = date.getMonth();
						date.setMonth(month);
					}
					else {
						date.setMonth(reminderArray[0]);
					}
					date.setDate(reminderArray[1]);
					
					var reminderSummary = reminderArray[2];
					
					var reminderEvent = new ReminderFoxEvent( date, reminderSummary );
					if ( reminderArray[0] == REMINDER_FOX_MONTHLY_WILDCARD_CHAR ) {
						reminderEvent.recurrence = REMINDER_RECURRENCE_MONTHLY_DATE;
					}				
					
				    // check for any options that are specified, and mark them in the columns 
		            for ( j = 3; j < reminderArray.length; j++ ) {
		                var val = reminderArray[j];
						 if ( val == REMINDER_IMPORTANT_FLAG ) {
		                    reminderEvent.priority = REMINDER_PRIORITY_IMPORTANT;
		                }	
		                else  if ( val == REMINDER_FOX_OCCURRENCE_ONCE ) {
		                    reminderEvent.recurrence = REMINDER_RECURRENCE_ONETIME;
		                }		
		                else  if ( val == REMINDER_TO_BE_COMPLETED_DELIMITER ) {
		                    reminderEvent.complete = true;
		                }	
		                else  if ( val.indexOf(REMINDER_TO_BE_COMPLETED_DELIMITER) == 0  ) {
		                	if (  val.indexOf(REMINDER_TO_BE_COMPLETED_DELIMITER, 1) == 1  ) { 
			                    reminderEvent.remindUntilCompleted = REMINDERFOX_REMIND_UNTIL_COMPLETE_MARKED;              
			                    val = val.substring(2, val.length );		                	
		                	}
		                	else {
			                    reminderEvent.remindUntilCompleted = REMINDERFOX_REMIND_UNTIL_COMPLETE_TO_BE_MARKED;			                    
			                    val = val.substring(1, val.length );
			                 }
		                    var remindCompletedArray = val.split("-");			
		                    reminderEvent.originalDate.setMonth(parseInt(remindCompletedArray[0]) );
		                    reminderEvent.originalDate.setDate(parseInt(remindCompletedArray[1]) );	                  
		                }							                			               
		            }
		            
		            reminderFoxEvents[i] = reminderEvent;
			}
		}
		
		/* TODOS  */
		
		var todos = "";
		try {
			todos = reminderFox_getUnicodePref(REMINDER_FOX_TODOS);
		}
		catch (e) {
		}
		
		if ( todos != "" ) {
			var todosArray = todos.split(REMINDER_SEPARATOR);
			for( i = 0; i < todosArray.length; i++) {
	
					var newTodo = todosArray[i];
					var todoArray = newTodo.split(REMINDER_PROPERTY_DELIMITER);							
					var todoSummary = todoArray[0];
					
					var todoEvent = new ReminderFoxTodo(  todoSummary );

					// check for any options that are specified, and mark them in the columns 
		            for ( j = 1; j < todoArray.length; j++ ) {
		                val = todoArray[j];
						 if ( val == REMINDER_IMPORTANT_FLAG ) {
		                    todoEvent.priority = REMINDER_PRIORITY_IMPORTANT;
		                }	
		                else  if ( val == REMINDER_TO_BE_COMPLETED_DELIMITER ) {
		                    todoEvent.showInTooltip = true;
		                }							                			               
		            }
		            
		            reminderFoxTodos[i] = todoEvent;							            
			}
		}
	}

	function reminderFox_getReminderEvents() {
		if ( reminderFoxEvents == null ) {
			reminderFox_readInRemindersAndTodos();
		}
		return reminderFoxEvents;
	}
	
	function reminderFox_getReminderTodos() {
		if ( reminderFoxTodos == null ) {
			reminderFox_readInRemindersAndTodos();
		}
		return reminderFoxTodos;
	}
	
	function reminderFox_clearRemindersAndTodos() {
		reminderFoxEvents = null;
		reminderFoxTodos = null;
		g_reminderFox_uniqueEventId = 0;
		g_reminderFox_uniqueTodoId = 0;
	}
	
	function reminderFox_getRemindersById( id ) {
		var reminders = reminderFox_getReminderEvents();
		for ( var i = 0; i < reminders.length; i++ ) {
			if ( reminders[i].id == id ) {
				return reminders[i];
			}
		}
		return null;
	}
	
	function reminderFox_getTodosById( id ) {
		var todos = reminderFox_getReminderTodos();
		for ( var i = 0; i < todos.length; i++ ) {
			if ( todos[i].id == id ) {
				return todos[i];
			}
		}
		return null;
	}

function reminderFox_isInteger(s)
{   var i;
    for (i = 0; i < s.length; i++)
    {   
        // Check that current character is number.
        var c = s.charAt(i);
        if (((c < "0") || (c > "9"))) return false;
    }
    // All characters are numbers.
    return true;
}
	
/*
 * check the reminder description for any variables (eg: <1978>) that needs translating
 */	
function reminderFox_processReminderDescription( reminder, year ) {
	var index = reminder.summary.indexOf( "<" );
	if ( index != -1 ) {
		var endIndex = reminder.summary.indexOf( ">", index + 1 );
		if ( endIndex != -1 ) {
			var yearVal = reminder.summary.substring( index + 1, endIndex );
			if ( yearVal.length == 4 ) {
				try {
					if (!reminderFox_isInteger(yearVal) ) {
						return reminder;
					}
					var yearInt = parseInt(yearVal );
					var currentYear = year;
					if ( currentYear == null ) {
						currentYear =  new Date().getFullYear();
					}
					var numOfYears = currentYear - yearInt;
					var numOfYearsStr = numOfYears + "";
					var length = numOfYearsStr.length;

					if ( numOfYearsStr.lastIndexOf("11") != -1 && numOfYearsStr.lastIndexOf("11") == length -2 ) {
						numOfYearsStr =numOfYearsStr + "th";
					}
					else if ( numOfYearsStr.lastIndexOf("12") != -1 && numOfYearsStr.lastIndexOf("12") == length - 2 ) {
						numOfYearsStr =numOfYearsStr + "th";
					}
					else if (  numOfYearsStr.lastIndexOf("13") != -1 && numOfYearsStr.lastIndexOf("13") == length -2 ) {
						numOfYearsStr =numOfYearsStr + "th";
					}											
					else if (  numOfYearsStr.lastIndexOf("1") != -1 &&numOfYearsStr.lastIndexOf("1") == length - 1) {
						numOfYearsStr =numOfYearsStr + "st";
					}
					else if (numOfYearsStr.lastIndexOf("2") != -1 && numOfYearsStr.lastIndexOf("2") == length -1  ) {
						numOfYearsStr =numOfYearsStr + "nd";
					}	
					else if ( numOfYearsStr.lastIndexOf("3") != -1 && numOfYearsStr.lastIndexOf("3") == length - 1 ) {
						numOfYearsStr =numOfYearsStr + "rd";
					}	
					else {
						numOfYearsStr =numOfYearsStr + "th";
					}											

					var returnReminder = reminderFox_cloneReminderFoxEvent(reminder);
					returnReminder.summary = returnReminder.summary.substring( 0, index ) + numOfYearsStr + 
						returnReminder.summary.substring( endIndex + 1 );
					return returnReminder;					
				}
				catch ( e ) {
				}
			}
		}
	}
	return reminder;
}	
